/*
 * CatalogViewer.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.app;

import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.io.*;
import java.util.*;
import misao.util.*;
import misao.pixy.*;

class CatalogViewer extends Frame implements Runnable {
    static final int window_size = 800;		// window size
    static final int total_height = window_size + 400;

    Thread thread = null;

    CatalogIndex catalog_index;
    Vector star_catalogs;
    Vector bgimage_catalogs;

    Panel panelA;
    Panel panelB;
    Panel panelC;
    Panel panelCatalog;
    Panel panelBgImage;
    Panel panelPosition;
    Panel panelArea;
    Panel panelView;

    CatalogCanvas catalog_canvas;
    Choice choiceCatalog;		// catalog list
    Choice choiceBgImage;		// background image list
    Checkbox checkCatalog;		// if plot catalog data (WWW)
    Checkbox checkBgImage;		// if draw background image
    TextField textCdrom;		// CD-ROM path
    TextField textRA_h;			// R.A. (h)
    TextField textRA_m;			// R.A. (m)
    TextField textRA_s;			// R.A. (s)
    CheckboxGroup checkDeclSgn;		// Sign of Decl.
    Checkbox checkDeclSgnP;		// Sign of Decl. (+)
    Checkbox checkDeclSgnM;		// Sign of Decl. (-)
    TextField textDecl_d;		// Decl. (o)
    TextField textDecl_m;		// Decl. (')
    TextField textDecl_s;		// Decl. (")
    TextField textFov;			// field of view
    TextField textMag;			// limiting magnitude
    TextField textPA;			// position angle of up direction
    Checkbox checkCenter;		// if mark the center
    Checkbox checkEastRight;		// if east is righthand
    Checkbox checkBgBlack;		// if background is black
    Checkbox checkB1950;		// if in B1950.0
    Button btnView;			// button to view

    CatalogList catalog_list;
    ImageRGB bg_image = null;

    boolean cache_if_Catalog = false;
    Catalog cache_Catalog = null;
    Catalog cache_BgImage = null;
    double cache_RA = -1.0;
    double cache_Decl = 99.9;
    double cache_fov_catalog = -1.0;
    double cache_fov_bgimage = -1.0;
    double cache_mag = 99.9;
    double cache_pa = -999.9;
    boolean cache_east_right = false;

    public static void main ( String[] args ) {
	CatalogViewer catalog_viewer = new CatalogViewer();
    }

    public CatalogViewer ( ) {
	init();
	start();

	resize(window_size, total_height);
	show();
    }

    public void init ( ) {
	catalog_index = new CatalogIndex();
	star_catalogs = new Vector();
	bgimage_catalogs = new Vector();

	catalog_canvas = new CatalogCanvas(this, new Size(window_size, window_size), new Position((double)window_size / 2.0, (double)window_size / 2.0));

	choiceCatalog = new Choice();
	choiceBgImage = new Choice();
	checkCatalog = new Checkbox("Catalog:");
	checkCatalog.setState(true);
	checkBgImage = new Checkbox("Background Image:");
	checkBgImage.setState(false);
	choiceBgImage.disable();
	textCdrom = new TextField("/cdrom", 10);

	textRA_h = new TextField("0", 2);
	textRA_m = new TextField("0", 2);
	textRA_s = new TextField("0.0", 5);
	checkDeclSgn = new CheckboxGroup();
	checkDeclSgnP = new Checkbox("+", checkDeclSgn, true);
	checkDeclSgnM = new Checkbox("-", checkDeclSgn, false);
	textDecl_d = new TextField("0", 2);
	textDecl_m = new TextField("0", 2);
	textDecl_s = new TextField("0.0", 4);
	textFov = new TextField("5.0", 4);
	textMag = new TextField("11.0", 4);
	textPA = new TextField("0.0", 5);

	checkCenter = new Checkbox("Mark the Center");
	checkEastRight = new Checkbox("East Righthand");
	checkBgBlack = new Checkbox("Black Background");
	checkB1950 = new Checkbox("B1950.0");

	checkCenter.setState(true);
	checkEastRight.setState(false);
	checkBgBlack.setState(false);
	checkB1950.setState(false);
	catalog_canvas.markCenter(checkCenter.getState());
	catalog_canvas.eastRight(checkEastRight.getState());
	catalog_canvas.bgBlack(checkBgBlack.getState());

	btnView = new Button("View");

	Catalog catalog = catalog_index.getFirstStarCatalog();
	while (catalog != null) {
	    choiceCatalog.addItem(catalog.getName());
	    star_catalogs.addElement(catalog);
	    catalog = catalog_index.getNextStarCatalog();
	}
	choiceCatalog.select(0);

	catalog = catalog_index.getFirstImageCatalog();
	while (catalog != null) {
	    choiceBgImage.addItem(catalog.getName());
	    bgimage_catalogs.addElement(catalog);
	    catalog = catalog_index.getNextImageCatalog();
	}
	choiceBgImage.select(0);

	layoutPanels();
    }

    private void layoutPanels ( ) {
	panelCatalog = new Panel();
	panelCatalog.setLayout(new FlowLayout());
	panelCatalog.add(checkCatalog);
	panelCatalog.add(choiceCatalog);
	panelCatalog.add(new Label("Path:"));
	panelCatalog.add(textCdrom);

	panelBgImage = new Panel();
	panelBgImage.setLayout(new FlowLayout());
	panelBgImage.add(checkBgImage);
	panelBgImage.add(choiceBgImage);

	panelPosition = new Panel();
	panelPosition.setLayout(new FlowLayout());
	panelPosition.add(new Label("RA:"));
	panelPosition.add(textRA_h);
	panelPosition.add(new Label("h"));
	panelPosition.add(textRA_m);
	panelPosition.add(new Label("m"));
	panelPosition.add(textRA_s);
	panelPosition.add(new Label("s  Decl:"));
	panelPosition.add(checkDeclSgnP);
	panelPosition.add(checkDeclSgnM);
	panelPosition.add(textDecl_d);
	panelPosition.add(new Label("o"));
	panelPosition.add(textDecl_m);
	panelPosition.add(new Label("'"));
	panelPosition.add(textDecl_s);
	panelPosition.add(new Label("\""));

	panelArea = new Panel();
	panelArea.setLayout(new FlowLayout());
	panelArea.add(new Label("Field:"));
	panelArea.add(textFov);
	panelArea.add(new Label("deg  Limit:"));
	panelArea.add(textMag);
	panelArea.add(new Label("mag  p.a. of up:"));
	panelArea.add(textPA);
	panelArea.add(new Label("deg"));
	panelArea.add(checkB1950);

	panelView = new Panel();
	panelView.setLayout(new FlowLayout());
	panelView.add(checkCenter);
	panelView.add(checkEastRight);
	panelView.add(checkBgBlack);
	panelView.add(btnView);

	panelA = new Panel();
	panelA.setLayout(new BorderLayout());
	panelA.add("North",  panelCatalog);
	panelA.add("Center", panelBgImage);

	panelB = new Panel();
	panelB.setLayout(new BorderLayout());
	panelB.add("North",  panelPosition);
	panelB.add("Center", panelArea);
	panelB.add("South",  panelView);

	panelC = new Panel();
	panelC.setLayout(new BorderLayout());
	panelC.add("North",  panelA);
	panelC.add("Center", panelB);

	setLayout(new BorderLayout());
	add("North",  panelC);
	add("Center", catalog_canvas);
    }
        
    public void start ( ) {
	thread = new Thread(this);
	thread.start();
    }

    public void stop() {
	if (thread != null) {
	    thread.stop();
	    thread = null;
	}
    }

    public boolean action(Event e, Object o) {
	if (e.target instanceof Button) {
	    if (btnView.getLabel().equals(o)) {
		try {
		    go();
		} catch ( Exception except ) {
		}
		return true;
	    }
	} else if (e.target instanceof Choice) {
	    Choice c = (Choice)(e.target);
	    double fov = (double)(Float.valueOf(textFov.getText()).floatValue());
	    double fov_max = 90.0;
	    if (c == choiceCatalog) {
		Catalog catalog = (Catalog)star_catalogs.elementAt(choiceCatalog.getSelectedIndex());
		fov_max = catalog.getFovLimit();

		if (catalog.isCdrom())
		    textCdrom.enable();
		else
		    textCdrom.disable();
	    } else if (c == choiceBgImage) {
		Catalog catalog = (Catalog)bgimage_catalogs.elementAt(choiceBgImage.getSelectedIndex());
		fov_max = catalog.getFovLimit();
	    }
	    if (fov > fov_max) {
		textFov = new TextField(Astro.toString(fov_max, 4, 2).trim(), 4);
		removeAll();
		layoutPanels();
		validate();
	    }
	    return true;
	} else if (e.target instanceof Checkbox) {
	    Checkbox c = (Checkbox)(e.target);
	    if (checkCatalog.getLabel().equals(c.getLabel())) {
		if (checkCatalog.getState()) {
		    choiceCatalog.enable();
		} else {
		    choiceCatalog.disable();
		}
		return true;
	    } else if (checkBgImage.getLabel().equals(c.getLabel())) {
		if (checkBgImage.getState()) {
		    choiceBgImage.enable();
		} else {
		    choiceBgImage.disable();
		}
		return true;
	    }
	}
	return false;
    }

    public void run() {
	while (true) {
	    try {
		Thread.sleep(500);
	    } catch (InterruptedException e) {
		System.err.println("sleep() is interrupted. " + e.getMessage());
		break;
	    }
	}
    }

    private void go ( ) throws Exception {
	Coor coor = new Coor();
	double fov = 0.0;
	double mag = 0.0;
	double pa = 0.0;
	try {
	    coor = new Coor(Integer.parseInt(textRA_h.getText().trim()), Integer.parseInt(textRA_m.getText().trim()), (double)(Float.valueOf(textRA_s.getText().trim()).floatValue()), checkDeclSgnM.getState(), Integer.parseInt(textDecl_d.getText().trim()), Integer.parseInt(textDecl_m.getText().trim()), (double)(Float.valueOf(textDecl_s.getText().trim()).floatValue()));
	    fov = (double)(Float.valueOf(textFov.getText().trim()).floatValue());
	    mag = (double)(Float.valueOf(textMag.getText().trim()).floatValue());
	    pa = (double)(Float.valueOf(textPA.getText().trim()).floatValue());
	} catch ( NumberFormatException e ) {
	    System.err.println("Number format exception: " + e.getMessage());
	    throw e;
	}

	double pa_tmp = Astro.norm(pa, 90.0);
	if (pa_tmp > 45.0)
	    pa_tmp = 90.0 - pa_tmp;
	double fov_rate = 1.0 / Math.cos(pa_tmp * Astro.RAD);
	double fov_catalog = fov * fov_rate;
	double fov_bgimage = fov * fov_rate;

	if (0.0 <= coor.getRA()  &&  coor.getRA() < 360.0  &&  -90.0 <= coor.getDecl()  &&  coor.getDecl() <= 90.0) {
	    // B1950.0
	    if (checkB1950.getState())
		coor.precession(new JulianDay(Astro.BESSELL_1950), new JulianDay(Astro.JULIUS_2000));

	    if (0.0 < fov  &&  fov <= 90.0) {
		Catalog catalog = (Catalog)star_catalogs.elementAt(choiceCatalog.getSelectedIndex());

		try {
		    double fov_max = 90.0;
		    fov_max = catalog.getFovLimit();
		    if (fov_catalog > fov_max)
			fov_catalog = fov_max;

		    if (cache_if_Catalog != checkCatalog.getState()  ||  cache_Catalog != catalog  ||  cache_RA != coor.getRA()  ||  cache_Decl != coor.getDecl()  ||  cache_fov_catalog != fov_catalog  ||  cache_mag != mag  ||  cache_pa != pa) {
			if (checkCatalog.getState()) {
			    catalog_list = catalog.createCatalogList();
			    if (catalog.isCdrom()) {
				StringTokenizer st = new StringTokenizer(textCdrom.getText());
				String[] root_dir = new String[st.countTokens()];
				int i = 0;
				while (st.hasMoreTokens())
				    root_dir[i++] = st.nextToken();
				((CdromList)catalog_list).load(coor, fov_catalog, mag, root_dir);
			    } else {
				((CgiList)catalog_list).load(coor, fov_catalog, mag);
			    }
			}

			catalog_list.rescaleXY((double)window_size * fov_catalog / fov);

			catalog_canvas.setCatalog(catalog_list, new MapFunction(1.0, - pa, new Position()), mag);
		    }
		} catch ( Exception e ) {
		    System.err.println("Cannot load catalog data: " + e.toString());
		}

		Catalog bgimage_catalog = (Catalog)bgimage_catalogs.elementAt(choiceBgImage.getSelectedIndex());
		try {
		    double fov_max = bgimage_catalog.getFovLimit();
		    if (fov_bgimage > fov_max)
			fov_bgimage = fov_max;

		    if (cache_BgImage != bgimage_catalog  ||  cache_RA != coor.getRA()  ||  cache_Decl != coor.getDecl()  ||  cache_fov_bgimage != fov_bgimage) {
			bg_image = null;
			if (checkBgImage.getState())
			    bg_image = bgimage_catalog.loadImage(coor, fov_bgimage);

			if (bg_image == null) {
			    catalog_canvas.clearImage();
			} else {
			    if (checkEastRight.getState())
				bg_image.mirrorHorizontal();
			    if (pa != 0.0) {
				bg_image = bg_image.rotate((checkEastRight.getState() ? pa : - pa));
				bg_image = bg_image.resize(new Size((int)((double)bg_image.getWidth() * fov / fov_bgimage), (int)((double)bg_image.getHeight() * fov / fov_bgimage)));
			    }
			    catalog_canvas.setImage(bg_image.rescale(new Size(window_size, window_size)).getImage());
			}
		    } else if (cache_east_right != checkEastRight.getState()) {
			bg_image.mirrorHorizontal();
			catalog_canvas.setImage(bg_image.rescale(new Size(window_size, window_size)).getImage());
		    }
		} catch ( Exception e ) {
		    System.err.println("Cannot load background image: " + e.toString());
		}

		catalog_canvas.markCenter(checkCenter.getState());
		catalog_canvas.eastRight(checkEastRight.getState());
		catalog_canvas.bgBlack(checkBgBlack.getState());

		catalog_canvas.updateOffScreen();
		catalog_canvas.repaint();

		cache_if_Catalog = checkCatalog.getState();
		cache_Catalog = catalog;
		cache_BgImage = bgimage_catalog;
		cache_RA = coor.getRA();
		cache_Decl = coor.getDecl();
		cache_fov_catalog = fov_catalog;
		cache_fov_bgimage = fov_bgimage;
		cache_mag = mag;
		cache_mag = pa;
		cache_east_right = checkEastRight.getState();
	    }
	}
    }
}
