/*
 * PastImageChecker.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.app;

import java.io.*;
import java.util.*;
import misao.util.*;
import misao.host.*;
import misao.pixy.*;

public class PastImageChecker {
    public static void main ( String[] args ) throws Exception {	
	int menu = 1;

	Coor coor = null;
	double limit_mag = 99.9;
	double area_shift = 0.0;
	
	Orbit orbit = new Orbit();

	boolean batch_mode = false;
	Vector coor_list = new Vector();

	if (args.length == 0) {
	    System.out.println("Output (x,y) of the star on the past images.");

	    DataInputStream din = new DataInputStream(System.in);

	    System.out.println("Menu: ");
	    System.out.println("  1. Star");
	    System.out.println("  2. Comet");
	    System.out.println("  3. Asteroid");
	    System.out.print("Menu: ");
	    menu = Integer.parseInt(din.readLine());

	    System.out.print("Area shift: ");
	    area_shift = Astro.doubleValueOf(din.readLine());

	    if (menu == 2) {
		System.out.println("Orbital elements: ");
		System.out.println("  T   : ");
		System.out.println("  ex. 1998 Jan. 31.12345");
		System.out.println("  acceptable style: 1998 Jan 31.12345");
		System.out.println("                    1998 1 31.12345");
		System.out.print("  T   : ");
		orbit.T = JulianDay.createJulianDay(din.readLine());
		System.out.print("  Peri: ");
		orbit.peri = Astro.doubleValueOf(din.readLine());
		System.out.print("  Node: ");
		orbit.node = Astro.doubleValueOf(din.readLine());
		System.out.print("  i   : ");
		orbit.i = Astro.doubleValueOf(din.readLine());
		System.out.print("  q   : ");
		orbit.q = Astro.doubleValueOf(din.readLine());
		System.out.print("  e   : ");
		orbit.e = Astro.doubleValueOf(din.readLine());

		System.out.print("Limiting mag: ");
		limit_mag = Astro.doubleValueOf(din.readLine());

		System.out.println("R.A.        Decl.        (x,y)               Lm.   Image");
	    } else if (menu == 3) {
		System.out.println("Orbital elements: ");
		System.out.println("  Epoch: ");
		System.out.println("  ex. 1998 Jan. 31.12345");
		System.out.println("  acceptable style: 1998 Jan 31.12345");
		System.out.println("                    1998 1 31.12345");
		System.out.print("  Epoch: ");
		orbit.T = JulianDay.createJulianDay(din.readLine());
		System.out.print("  M    : ");
		orbit.M = Astro.doubleValueOf(din.readLine());
		System.out.print("  Peri : ");
		orbit.peri = Astro.doubleValueOf(din.readLine());
		System.out.print("  Node : ");
		orbit.node = Astro.doubleValueOf(din.readLine());
		System.out.print("  i    : ");
		orbit.i = Astro.doubleValueOf(din.readLine());
		System.out.print("  a    : ");
		orbit.q = Astro.doubleValueOf(din.readLine());
		System.out.print("  e    : ");
		orbit.e = Astro.doubleValueOf(din.readLine());

		orbit.convertAsteroidToComet();

		System.out.print("Limiting mag: ");
		limit_mag = Astro.doubleValueOf(din.readLine());

		System.out.println("R.A.        Decl.        (x,y)               Lm.   Image");
	    } else {
		System.out.println("Positions (J2000.0): ");
		System.out.println("  ex. R.A.= 12h 4m31.2s  Decl.= - 0o15'");
		System.out.println("  acceptable style: 12 4 31.2 -0 15 0");
		System.out.print("Positions (J2000.0): ");
		String position = din.readLine();
		coor = Coor.createCoor(position);

		System.out.print("Limiting mag: ");
		limit_mag = Astro.doubleValueOf(din.readLine());

		System.out.println("(x,y)               Lm.   Image");
	    }
	} else {
	    String option = args[0];

	    if (option.equals("-c")) {
		menu = 2;

		orbit.T = JulianDay.createJulianDay(args[1]);
		orbit.peri = Astro.doubleValueOf(args[2]);
		orbit.node = Astro.doubleValueOf(args[3]);
		orbit.i = Astro.doubleValueOf(args[4]);
		orbit.q = Astro.doubleValueOf(args[5]);
		orbit.e = Astro.doubleValueOf(args[6]);
		limit_mag = Astro.doubleValueOf(args[7]);

		System.out.println("R.A.        Decl.        (x,y)               Lm.   Image");
	    } else if (option.equals("-a")) {
		menu = 3;

		orbit.T = JulianDay.createJulianDay(args[1]);
		orbit.M = Astro.doubleValueOf(args[2]);
		orbit.peri = Astro.doubleValueOf(args[3]);
		orbit.node = Astro.doubleValueOf(args[4]);
		orbit.i = Astro.doubleValueOf(args[5]);
		orbit.q = Astro.doubleValueOf(args[6]);
		orbit.e = Astro.doubleValueOf(args[7]);

		orbit.convertAsteroidToComet();

		limit_mag = Astro.doubleValueOf(args[8]);

		System.out.println("R.A.        Decl.        (x,y)               Lm.   Image");
	    } else {
		int arg_base = 0;
		if (option.equals("-s"))
		    arg_base = 1;

		limit_mag = Astro.doubleValueOf(args[arg_base + 1]);

		try {
		    batch_mode = true;

		    File file = new File(args[arg_base]);
		    FileInputStream fis = new FileInputStream(file);
		    DataInputStream dis = new DataInputStream(new BufferedInputStream(fis));
		    String s = dis.readLine();
		    while (s != null) {
			coor = Coor.createCoor(s);
			coor_list.addElement(coor);
			s = dis.readLine();
		    }
		    dis.close();
		} catch (Exception e) {
		    batch_mode = false;
		    coor = Coor.createCoor(args[arg_base]);
		}
	    }
	}

	int loop_max = 1;
	if (batch_mode)
	    loop_max = coor_list.size();

	for (int loop = 0 ; loop < loop_max ; loop++) {
	    if (batch_mode) {
		coor = (Coor)coor_list.elementAt(loop);
		System.out.println("\n");
		System.out.println("Position: " + coor.toString());
	    }

	    ImageInformationList image_info_list = new ImageInformationList();

	    File file = new File(MisaoProperty.getImageInformationListPath());
	    FileInputStream fis = new FileInputStream(file);
	    DataInputStream dis = new DataInputStream(new BufferedInputStream(fis));

	    ImageInformation image_info = image_info_list.readNextImageInformation(dis);
	    while (image_info != null) {
		if (image_info.limiting_mag > limit_mag) {
		    if (menu == 2  ||  menu == 3) {
			JulianDay jd = new JulianDay(image_info.exposure_start.getJD() + image_info.exposure_seconds / 2.0 / 3600.0 / 24.0);
			OrbitPosition orbit_position = orbit.calculatePosition(jd);
			coor = orbit_position.coor;
		    }

		    Position pos = new CoorMapping(coor).mappingXY(image_info.center_coor);
		    pos.rescale((double)image_info.image_size.getWidth() / image_info.field_of_view);
		    pos = new MapFunction(1.0, 360.0 - image_info.pa, new Position()).inverse().map(pos);
		    Position df = image_info.distortion_field.getValue(pos);
		    df.rescale(-1.0);
		    pos.add(df);
		    pos.add(new Position((double)image_info.image_size.getWidth() / 2.0, (double)image_info.image_size.getHeight() / 2.0));

		    Position pos2 = new Position(pos);
		    pos2.add(new Position(- (double)image_info.image_size.getWidth() / 2.0, - (double)image_info.image_size.getHeight() / 2.0));
		    pos2.rescale(1.0 / (double)image_info.image_size.getWidth());
		    if (Math.abs(pos2.getX()) <= 0.5) {
			pos2.setX(0.0);
		    } else {
			if (pos2.getX() < 0)
			    pos2.add(new Position(0.5,0));
			else
			    pos2.add(new Position(-0.5,0));
		    }
		    if (Math.abs(pos2.getY()) <= 0.5) {
			pos2.setY(0.0);
		    } else {
			if (pos2.getY() < 0)
			    pos2.add(new Position(0,0.5));
			else
			    pos2.add(new Position(0,-0.5));
		    }

		    if (0 <= pos.getX()  &&  pos.getX() < image_info.image_size.getWidth()  &&  0 <= pos.getY()  &&  pos.getY() < image_info.image_size.getHeight()) {
			if (menu == 2  ||  menu == 3)
			    System.out.print(coor.toStringRA() + " " + coor.toStringDecl() + "  ");

			String record = "(";
			record += Astro.toString(pos.getX(), 7, 4);
			record += ", ";
			record += Astro.toString(pos.getY(), 7, 4);
			record += ")  ";
			record += Astro.toString(image_info.limiting_mag, 4, 2);
			record += "  ";
			record += image_info.image_url;

			System.out.println(record);
		    } else if (Math.abs(pos2.getX()) <= area_shift  &&  Math.abs(pos2.getY()) <= area_shift) {
			if (menu == 2  ||  menu == 3)
			    System.out.print(coor.toStringRA() + " " + coor.toStringDecl() + "  ");

			String record = "[";
			record += Astro.toString(pos2.getX(), 6, 4);
			record += " , ";
			record += Astro.toString(pos2.getY(), 6, 4);
			record += " ]  ";
			record += Astro.toString(image_info.limiting_mag, 4, 2);
			record += "  ";
			record += image_info.image_url;

			System.out.println(record);
		    }

		    // print data in PXF file around the (x,y) position
		    if (0 <= pos.getX()  &&  pos.getX() < image_info.image_size.getWidth()  &&  0 <= pos.getY()  &&  pos.getY() < image_info.image_size.getHeight()) {
			PxfFile pxf = PxfFile.readPxfFileOfImage(image_info.image_url);
			if (pxf != null) {
			    Vector list = pxf.pair_list.getAllList();
			    for (int k = 0 ; k < list.size() ; k++) {
				Vector l = (Vector)list.elementAt(k);

				for (int j = 0 ; j < l.size() ; j++) {
				    PairData pair = (PairData)l.elementAt(j);
				    StarData sdd = (StarData)pair.getFirstData();
				    StarData sdc = (StarData)pair.getSecondData();
				    StarData sd = sdc;
				    if (sd == null)
					sd = sdd;
				    Position p = sd.getPosition();
				    if (sdd != null)
					p = sdd.getPosition();

				    if (Math.abs(p.getX() - pos.getX()) < 3.0  &&  Math.abs(p.getY() - pos.getY()) < 3.0) {
					System.out.println(pair.getName());
					Vector alias_list = pair.getAliasList();
					for (int n = 0 ; n < alias_list.size() ; n++) {
					    ReferenceName name = (ReferenceName)alias_list.elementAt(n);
					    System.out.println(name.getType().toString() + "=" + name.getName());
					}
					pair.print(System.out);
					System.out.println("");
				    }
				}
			    }
			}
		    }
		}

		image_info = image_info_list.readNextImageInformation(dis);
	    }

	    dis.close();
	}
    }
}
