/*
 * FITSHeadEnv.java
 *
 * Copyright (C) 1998-2000 Taichi Kato
 *
 * Original package is jp.ac.kyoto-u.kusastro.ooruri.
 *
 */

package misao.contrib.imdisp;

import java.io.*;

public class FITSHeadEnv {
	private DataInputStream ds;
	private FITSHead head;
	private String object;
	private int pixtype;
	private boolean reverse;
	private boolean unsigned;
	private int w;
	private int h;
	private double bzero;
	private double bscale;

	public FITSHeadEnv(DataInputStream _ds) {
		ds = _ds;
		init(true);
	}

	public FITSHeadEnv(DataInputStream _ds,boolean printHeader) {
		ds = _ds;
		init(printHeader);
	}

	private void init(boolean printHeader) {
		head = new FITSHead(ds);
		int naxis = head.getInt("NAXIS");
		if (naxis > 3 || (naxis == 3 && head.getInt("NAXIS3") != 1))
			return;
		int bitpix = head.getInt("BITPIX");
		if (bitpix == 16)
			pixtype = 1;
		else if (bitpix == 32)
			pixtype = 2;
		else if (bitpix == -32)
			pixtype = 3;
		else if (bitpix == 8)
			pixtype = 4;
		else
			return;
		w = head.getInt("NAXIS1");
		h = head.getInt("NAXIS2");
		if (h*w <= 0)
			return;
		bzero = head.getDouble("BZERO");
		bscale = head.getDouble("BSCALE");
		if (bscale == 0)
			bscale = 1;
		if (printHeader)
			System.out.println(head.allString());
			String obj = head.getString("OBJECT");
		if (obj != null && obj.startsWith("DSS"))
			reverse = true;
		String sign = head.getString("UNSIGNED");
		if (sign != null && sign.equals("T"))
			unsigned = true;
	}

	public int width() {
		return w;
	}

	public int height() {
		return h;
	}

	public FITSHead getHead() {
		return head;
	}

	public int trueY(int y) {
		if (!reverse)
			return y;
		else
			return h-y-1;
	}

	public double getNextPixel() {
		double val = 0;
		try {
			if (pixtype == 1) {
				if (unsigned)
					val = ds.readUnsignedShort();
				else
					val = ds.readShort();
			}	
			else if (pixtype == 2) {
				val = ds.readInt();
			}
			else if (pixtype == 3) {
				val = ds.readFloat();
			}
			else if (pixtype == 4) {
				val = ds.readUnsignedByte();
			}
		}
		catch (IOException e) {
			System.out.println("Exception: " + e.toString());
		}

		return val*bscale + bzero;
	}
}
