/*
 * MagnitudeDatabaseRecord.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.database;

import misao.util.*;
import misao.pixy.*;

public class MagnitudeDatabaseRecord {
    private JulianDay jd;
    private String mag;
    private String flag;
    private String image_id;
    private String name;
    private String xy;
    private String memo;

    public MagnitudeDatabaseRecord ( String s ) {
	int p = s.indexOf('\t');
	jd = JulianDay.createJulianDay(s.substring(0,p));

	s = s.substring(p+1);
	p = s.indexOf('\t');
	mag = s.substring(0,p);

	s = s.substring(p+1);
	p = s.indexOf('\t');
	flag = s.substring(0,p);

	s = s.substring(p+1);
	p = s.indexOf('\t');
	image_id = s.substring(0,p);

	s = s.substring(p+1);
	p = s.indexOf('\t');
	name = s.substring(0,p);

	s = s.substring(p+1);
	p = s.indexOf('\t');
	if (p >= 0) {
	    xy = s.substring(0,p);

	    s = s.substring(p+1);
	    memo = s;
	} else {
	    xy = s;
	    memo = "";
	}
    }

    public MagnitudeDatabaseRecord ( PairData pair, ImageInformation image_info ) {
	jd = new JulianDay(image_info.exposure_start.getJD() + image_info.exposure_seconds / 2.0 / 3600.0 / 24.0);

	mag = "";
	xy = "";
	try {
	    if (pair == null) {
		mag = "<" + Astro.toString(image_info.limiting_mag, 4, 2).trim();
	    } else if (pair.getType() == PairType.NC  ||  pair.getType() == PairType.NR) {
		mag = "<" + Astro.toString(image_info.limiting_mag, 4, 2).trim();
		xy = "(" + Astro.toString(pair.getFirstStarData().getX(), 7, 4) + "," + Astro.toString(pair.getFirstStarData().getY(), 7, 4) + ")";
	    } else {
		mag = Astro.toString(pair.getFirstMag(), 4, 2).trim();
		xy = "(" + Astro.toString(pair.getFirstStarData().getX(), 7, 4) + "," + Astro.toString(pair.getFirstStarData().getY(), 7, 4) + ")";
	    }
	} catch (Exception e) {
	}
	mag += image_info.mag_code;

	flag = "";

	image_id = image_info.image_url;

	name = "";
	if (pair != null)
	    name = pair.getName();

	memo = "";
	if (pair != null) {
	    if (pair.getType() != PairType.NC  &&  pair.getType() != PairType.NR) {
		if (pair.isBlending())
		    memo += "blending";
	    }
	    if (pair.getType() == PairType.PS  ||  pair.getType() == PairType.RM  ||  pair.getType() == PairType.RP  ||  pair.getType() == PairType.ND) {
		DetectData sd_detect = (DetectData)pair.getFirstData();

		if (sd_detect.isEdge()) {
		    if (memo.length() > 0)
			memo += ", ";
		    memo += "edge";
		}
	    }
	}
    }

    public JulianDay getJD ( ) {
	return jd;
    }

    public String getImage ( ) {
	return image_id;
    }

    public void setImage ( String s ) {
	image_id = s;
    }

    public double getMag ( ) {
	if (mag.charAt(0) == '<'  ||  mag.charAt(0) == '>'  ||  mag.charAt(0) == '['  ||  mag.charAt(0) == ']')
	    return Astro.doubleValueOf(mag.substring(1));

	return Astro.doubleValueOf(mag);
    }

    public boolean checkFlag ( char f ) {
	for (int i = 0 ; i < flag.length() ; i++) {
	    if (flag.charAt(i) == f)
		return true;
	}

	return false;
    }

    public boolean isDetected ( ) {
	if (mag.charAt(0) == '<'  ||  mag.charAt(0) == '>'  ||  mag.charAt(0) == '['  ||  mag.charAt(0) == ']')
	    return false;

	return true;
    }

    public double getX ( ) {
	if (xy.length() == 0)
	    return 0.0;

	return Astro.doubleValueOf(xy.substring(1));
    }

    public double getY ( ) {
	if (xy.length() == 0)
	    return 0.0;

	int p = xy.indexOf(',');
	return Astro.doubleValueOf(xy.substring(p+1));
    }

    public void setPosition ( Position pos ) {
	xy = "(" + Astro.toString(pos.getX(), 7, 4) + "," + Astro.toString(pos.getY(), 7, 4) + ")";
    }

    public String getMemo ( ) {
	return memo;
    }

    public void appendMemo ( String s ) {
	if (memo.length() > 0)
	    memo += ", ";
	memo += s;
    }

    public String getRecord ( ) {
	String date = "" + jd.getYear() + " " + jd.getMonthCode().substring(0, 3) + " " + Astro.toString(jd.getDate(), 8, 2);

	String record = date + "\t" + mag + "\t" + flag + "\t" + image_id + "\t" + name + "\t" + xy;
	if (memo.length() > 0)
	    record += "\t" + memo;

	return record;
    }
}
