/*
 * MagnitudeDatabaseStar.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.database;

import java.io.*;
import java.util.*;
import misao.util.*;
import misao.pixy.*;
import misao.host.*;

public class MagnitudeDatabaseStar implements MapElement, Printable {
    private ReferenceName ref_name;
    private Vector record_list;
    private int record_index;

    public MagnitudeDatabaseStar ( ReferenceName _ref_name ) {
	ref_name = _ref_name;
	record_list = new Vector();
	record_index = 0;
    }

    public boolean empty ( ) {
	if (record_list == null  ||  record_list.size() == 0)
	    return true;
	return false;
    }

    public String getName ( ) {
	return ref_name.getName();
    }

    public Position getPosition ( ) {
	Coor coor = new Coor();
	if (IrasObject.isIrasObject(ref_name)) {
	    coor = IrasObject.getCoor(ref_name);
	} else if (ref_name instanceof MisaoPreliminaryObject) {
	    MisaoPreliminaryObject o = (MisaoPreliminaryObject)ref_name;
	    coor = o.getCoor();
	}
	return new Position(coor.getRA(), coor.getDecl());
    }

    public void setReferenceName ( ReferenceName _ref_name ) {
	ref_name = _ref_name;
    }

    public ReferenceName getReferenceName ( ) {
	return ref_name;
    }

    public int countRecord ( ) {
	return record_list.size();
    }

    public MagnitudeDatabaseRecord getFirstRecord ( ) {
	record_index = 0;
	return getNextRecord();
    }

    public MagnitudeDatabaseRecord getNextRecord ( ) {
	if (record_index >= record_list.size())
	    return null;
	record_index++;
	return (MagnitudeDatabaseRecord)record_list.elementAt(record_index-1);
    }

    public void removeRecord ( MagnitudeDatabaseRecord new_record ) {
	for (int n = 0 ; n < record_list.size() ; n++) {
	    MagnitudeDatabaseRecord mag_record = (MagnitudeDatabaseRecord)record_list.elementAt(n);
	    if (new_record.getImage().equals(mag_record.getImage())) {
		record_list.removeElementAt(n);
		n--;
	    }
	}
    }

    public void addRecord ( MagnitudeDatabaseRecord new_record ) {
	removeRecord(new_record);

	for (int n = 0 ; n < record_list.size() ; n++) {
	    MagnitudeDatabaseRecord mag_record = (MagnitudeDatabaseRecord)record_list.elementAt(n);
	    if (new_record.getJD().getJD() < mag_record.getJD().getJD()) {
		record_list.insertElementAt(new_record, n);
		return;
	    }
	}
	record_list.addElement(new_record);
    }

    public MagnitudeDatabaseRecord getRecordOfImage ( String image ) {
	for (int n = 0 ; n < record_list.size() ; n++) {
	    MagnitudeDatabaseRecord mag_record = (MagnitudeDatabaseRecord)record_list.elementAt(n);
	    if (mag_record.getImage().equals(image))
		return mag_record;
	}
	return null;
    }

    public void read ( DataInputStream dis ) throws Exception {
	record_list = new Vector();

	String record = dis.readLine();
	while (record != null) {
	    MagnitudeDatabaseRecord mag_record = new MagnitudeDatabaseRecord(record);
	    record_list.addElement(mag_record);
	    record = dis.readLine();
	}
    }

    public void print ( PrintStream ps ) {
	for (int n = 0 ; n < record_list.size() ; n++) {
	    MagnitudeDatabaseRecord mag_record = (MagnitudeDatabaseRecord)record_list.elementAt(n);
	    ps.println(mag_record.getRecord());
	}
    }
}
