/*
 * Updater.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.database;

import java.io.*;
import java.util.*;
import misao.util.*;
import misao.pixy.*;
import misao.host.*;

public class Updater {
    private PxfFile pxf;
    private String image_id;

    private boolean to_delete;

    public Updater ( PxfFile _pxf ) {
	pxf = _pxf;

	to_delete = false;
    }

    public void setImageID ( String id ) {
	image_id = id;
    }

    public void setToDelete ( ) {
	to_delete = true;
    }

    public static void createDirectory ( ) {
	MisaoProperty.createObsDirectory();
	File directory = new File(MisaoProperty.getObsPath() + "/variable");
	if (! directory.isDirectory())
	    directory.mkdir();
	directory = new File(MisaoProperty.getObsPath() + "/suspicious");
	if (! directory.isDirectory())
	    directory.mkdir();
	directory = new File(MisaoProperty.getObsPath() + "/error");
	if (! directory.isDirectory())
	    directory.mkdir();
    }

    public void updateImageInformationList ( ) throws Exception {
	ImageInformationList image_info_list = new ImageInformationList();

	File file = new File(MisaoProperty.getImageInformationListPath());
	if (file.canRead()) {
	    FileInputStream fis = new FileInputStream(file);
	    DataInputStream dis = new DataInputStream(new BufferedInputStream(fis));
	    image_info_list.read(dis);
	    dis.close();
	}

	image_info_list.add(pxf.image_info);

	FileOutputStream os = new FileOutputStream(MisaoProperty.getImageInformationListPath());
	PrintStream ps = new PrintStream(os);
	image_info_list.print(ps);
	ps.close();
    }

    public void operate ( ) throws Exception {
	updateImageInformationList();

	MagnitudeDatabaseManager mag_db_manager = new MagnitudeDatabaseManager();
	if (to_delete)
	    mag_db_manager.setToDelete();

	Vector list = pxf.pair_list.getAllList();
	for (int i = 0 ; i < list.size() ; i++) {
	    Vector l = (Vector)list.elementAt(i);

	    for (int j = 0 ; j < l.size() ; j++) {
		PairData pair = (PairData)l.elementAt(j);
		DetectData sd_detect = (DetectData)pair.getFirstData();

		MagnitudeDatabaseRecord mag_record = new MagnitudeDatabaseRecord(pair, pxf.image_info);

		Vector alias_list = pair.getAliasList();

		boolean variable_flag = false;
		boolean suspicious_flag = false;
		boolean error_flag = false;
		boolean nebula_flag = false;
		boolean misao_preliminary_flag = false;
		for (int k = 0 ; k < alias_list.size() ; k++) {
		    ReferenceName ref_name = (ReferenceName)alias_list.elementAt(k);
		    if (ref_name.getType() == ReferenceType.REF_V) {
			variable_flag = true;
		    } else if (ref_name.getType() == ReferenceType.REF_S) {
			suspicious_flag = true;
		    } else if (ref_name.getType() == ReferenceType.REF_E) {
			error_flag = true;
		    } else if (ref_name.getType() == ReferenceType.REF_N) {
			nebula_flag = true;
		    }
		}

		if (nebula_flag) {
		    // do not accumulate magnitude data
		} else if (variable_flag) {
		    for (int k = 0 ; k < alias_list.size() ; k++) {
			ReferenceName ref_name = (ReferenceName)alias_list.elementAt(k);
			if (ref_name.getType() == ReferenceType.REF_V)
			    mag_db_manager.updateRecord(ReferenceType.REF_V, ref_name, mag_record);
		    }
		} else if (suspicious_flag) {
		    for (int k = 0 ; k < alias_list.size() ; k++) {
			ReferenceName ref_name = (ReferenceName)alias_list.elementAt(k);
			if (ref_name.getType() == ReferenceType.REF_S)
			    mag_db_manager.updateRecord(ReferenceType.REF_S, ref_name, mag_record);
		    }
		} else if (error_flag) {
		    for (int k = 0 ; k < alias_list.size() ; k++) {
			ReferenceName ref_name = (ReferenceName)alias_list.elementAt(k);
			if (ref_name.getType() == ReferenceType.REF_E)
			    mag_db_manager.updateRecord(ReferenceType.REF_E, ref_name, mag_record);
		    }
		} else if (pair.getType() == PairType.ND) {
		    double pos_error = (pxf.image_info.pos_error.getRA() + pxf.image_info.pos_error.getDecl()) / 2.0 * 3.0;
		    if (pos_error < 5.0)
			pos_error = 5.0;
		    MisaoPreliminaryObject o = new MisaoPreliminaryObject(sd_detect.getCoor(), pos_error);
		    mag_db_manager.updateRecord(ReferenceType.REF_S, o, mag_record);
		} else if (pair.getType() == PairType.NC) {
		    CatalogData sd_catalog = (CatalogData)pair.getSecondData();
		    MisaoPreliminaryObject o = new MisaoPreliminaryObject(sd_catalog.getCoor(), 5.0);
		    mag_db_manager.updateRecord(ReferenceType.REF_S, o, mag_record);

		    String old_id = sd_catalog.getID();
		    CatalogData sd = (CatalogData)sd_catalog.getFirstAppendix();
		    while (sd != null) {
			if (! sd.getID().equals(old_id)) {
			    o = new MisaoPreliminaryObject(sd.getCoor(), 5.0);
			    mag_db_manager.updateRecord(ReferenceType.REF_S, o, mag_record);
			}
			old_id = sd.getID();
			sd = (CatalogData)sd_catalog.getNextAppendix();
		    }
		}
	    }
	}

	mag_db_manager.close();
    }
}
