/*
 * DifferentialGalleryCanvas.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.diff;

import java.awt.*;
import java.util.Vector;
import misao.util.*;
import misao.pixy.*;

public class DifferentialGalleryCanvas extends Canvas {
    private DifferentialPixyOutputData output_data;
    private ImageGray gallery;
    private boolean image_created;

    private Dimension offSize;
    private Graphics offScreen;
    private Image offImage;

    public DifferentialGalleryCanvas ( DifferentialPixyOutputData _output_data ) {
	output_data = _output_data;

	gallery = null;
	image_created = false;

	DifferentialCheckImage check_image = new DifferentialCheckImage(output_data);
	Size size = check_image.getGallerySize();
	resize(size.getWidth(), size.getHeight());
    }

    public ImageGray getImage ( ) {
	if (image_created)
	    return gallery;
	return null;
    }

    public void updateOffScreen ( ) {
	DifferentialCheckImage check_image = new DifferentialCheckImage(output_data);
	Size size = check_image.getGallerySize();

	offScreen.setColor(Color.black);
	offScreen.fillRect(0, 0, size.getWidth(), size.getHeight());

	offScreen.setFont(new Font("TimesRoman", Font.BOLD, check_image.getFontSize()));
	offScreen.setColor(Color.white);

	int total_base_y = check_image.getFontSize() + check_image.getFontSpace();
	PairData pair = check_image.getFirstPairData();
	while (pair != null) {
	    offScreen.drawString(pair.getName(), 0, total_base_y);
	    total_base_y += check_image.getImageSize().getHeight() + check_image.getFontSize() + check_image.getFontSpace();
	    pair = check_image.getNextPairData();
	}

	if (gallery == null) {
	    try {
		gallery = new ImageGray(offImage, check_image.getGallerySize());
		total_base_y = check_image.getFontSize() + check_image.getFontSpace();
		check_image.prepareDrawImage();
		double mean = -1;
		pair = check_image.getFirstPairData();
		while (pair != null) {
		    if (mean < 0) {
			int count = 0;
			for (int y = total_base_y - check_image.getFontSize() ; y < total_base_y ; y++) {
			    for (int x = 0 ; x < check_image.getImageSize().getWidth() ; x++) {
				mean += gallery.getValue(new Point(x, y));
				count++;
			    }
			}
			mean /= (double)count;
		    }
		    for (int y = total_base_y - check_image.getFontSize() ; y < total_base_y ; y++) {
			for (int x = 0 ; x < check_image.getImageSize().getWidth() ; x++) {
			    double f = gallery.getValue(new Point(x, y));
			    if (f < mean)
				gallery.setValue(new Point(x, y), 0);
			    else
				gallery.setValue(new Point(x, y), check_image.getMaxValue());
			}
		    }

		    check_image.drawImage(gallery, pair, 0, total_base_y);
		    total_base_y += check_image.getImageSize().getHeight() + check_image.getFontSize() + check_image.getFontSpace();
		    pair = check_image.getNextPairData();
		}

		offImage = gallery.getImage();
	    } catch (Exception e) {
	    }

	    image_created = true;
	}
    }

    public void paint ( Graphics g ) {
	update(g);
    }

    public void update ( Graphics g ) {
	Dimension dim = size();
	if (offScreen == null  ||  dim.width != offSize.width ||  dim.height != offSize.height) {
	    offSize = dim;
	    offImage = createImage(dim.width, dim.height);
	    offScreen = offImage.getGraphics();

	    updateOffScreen();
	}

	g.drawImage(offImage, 0, 0, this);
    }
}
