/*
 * ReferenceCatalogBase.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.host;

import java.io.*;
import java.util.*;
import misao.pixy.*;
import misao.util.*;

public class ReferenceCatalogBase {
    private String path;
    private Vector file_list;
    private DataInputStream current_dis;
    private int current_file;

    public ReferenceCatalogBase ( ) {
	file_list = new Vector();
	current_dis = null;
	current_file = 0;

	path = MisaoProperty.getReferencePath();
    }

    public Vector findData ( Coor coor, double r ) {
	Vector list = new Vector();

	ReferenceData data = findFirstData(coor, r);
	while (data != null) {
	    list.addElement(data);
	    data = findNextData(coor, r);
	}

	return list;
    }

    public ReferenceData findFirstData ( Coor coor, double r ) {
	file_list = getFileList(coor, r);
	current_dis = null;
	current_file = 0;

	return findNextData(coor, r);
    }

    public ReferenceData findNextData ( Coor coor, double r ) {
	if (current_dis == null) {
	    if (current_file >= file_list.size())
		return null;

	    try {
		String filename = (String)file_list.elementAt(current_file);
		File file = new File(filename);
		FileInputStream fis = new FileInputStream(file);
		current_dis = new DataInputStream(new BufferedInputStream(fis));
	    } catch (IOException e) {
		return null;
	    }
	}

	while (true) {
	    String s = "";
	    try {
		s = current_dis.readLine();
		if (s == null) {
		    current_dis = null;
		    current_file++;
		    return findNextData(coor, r);
		}

		StringTokenizer st = new StringTokenizer(s, "\t");
		String name = st.nextToken();
		String dbvs_name = st.nextToken();
		double ra = Float.valueOf(st.nextToken()).floatValue();
		double decl = Float.valueOf(st.nextToken()).floatValue();
		double pos_error = Float.valueOf(st.nextToken()).floatValue();
		ReferenceType type = ReferenceType.getType(st.nextToken().charAt(0));
		String record = current_dis.readLine();

		Coor cr = new Coor(ra, decl);
		if (cr.angular_distance(coor) < r) {
		    ReferenceData data = new ReferenceData(name, dbvs_name, cr, record, pos_error, type);
		    return data;
		}
	    } catch (IOException e) {
		return null;
	    }
	}
    }

    public void replaceData ( ReferenceData data ) {
	String filename = getFile(data.getCoor());

	Vector string_list = new Vector();

	try {
	    boolean replaced_flag = false;

	    File file = new File(filename);
	    FileInputStream fis = new FileInputStream(file);
	    DataInputStream dis = new DataInputStream(new BufferedInputStream(fis));
	    String s = dis.readLine();
	    while (s != null) {
		String name = s.substring(0, s.indexOf('\t'));
		if (name.equals(data.getName())) {
		    s = data.toString();
		    string_list.addElement(s + "\t" + Astro.toString(data.getPositionError(), 5, 3).trim() + "\t" + data.getType().toString());
		    dis.readLine();
		    string_list.addElement(data.getRecord());

		    replaced_flag = true;
		} else {
		    string_list.addElement(s);
		    s = dis.readLine();
		    string_list.addElement(s);
		}
		s = dis.readLine();
	    }
	    dis.close();

	    if (replaced_flag == false) {
		string_list.addElement(data.toString() + "\t" + Astro.toString(data.getPositionError(), 5, 3).trim() + "\t" + data.getType().toString());
		string_list.addElement(data.getRecord());
	    }
	} catch (IOException e) {
	    System.err.println("new file: " + filename);
	    string_list.addElement(data.toString());
	    string_list.addElement(data.getRecord());
	}

	try {
	    FileOutputStream os = new FileOutputStream(filename);
	    PrintStream ps = new PrintStream(os);
	    for (int i = 0 ; i < string_list.size() ; i++)
		ps.println((String)string_list.elementAt(i));
	    ps.close();
	} catch (IOException e) {
	    System.err.println("cannot write file: " + filename);
	}
    }

    public String getFile ( Coor coor ) {
	int ra = (int)(coor.getRA() / 15.0);
	int decl = ((int)(coor.getDecl() + 90.0) / 10 - 9) * 10;

	return path + "/" + Astro.toStringZeroPadding(ra, 2) + (decl < 0 ? '-' : '+') + Astro.toStringZeroPadding(Math.abs(decl), 2);
    }

    public Vector getFileList ( Coor coor, double r ) {
	Vector file_list = new Vector();

	AreaManager area_manager = new AreaManager(coor, r);

	for (int ra = 0 ; ra < 24 ; ra++) {
	    for (int decl = -90 ; decl < 90 ; decl += 10) {
		Coor cr1 = new Coor((double)ra * 15.0, (double)decl);
		Coor cr2 = new Coor((double)(ra + 1) * 15.0, (double)(decl + 10));
		if (area_manager.overlapArea(cr1, cr2)) {
		    String file = path + "/" + Astro.toStringZeroPadding(ra, 2) + (decl < 0 ? '-' : '+') + Astro.toStringZeroPadding(Math.abs(decl), 2);
		    file_list.addElement(file);
		}
	    }
	}

	return file_list;
    }
}
