/*
 * AdjoiningPixel.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.*;
import java.util.Vector;
import misao.util.*;

public class AdjoiningPixel {
    private Vector point_list;
    private Vector value_list;

    private boolean edge_flag;

    // stack in construct()
    private ImagePixy construct_image;

    // stack in extract_recurrent()
    private AdjoiningPixel extract_ap;
    private int extract_index;
    private double extract_threshold;

    private AdjoiningPixel ( ) {
	point_list = new Vector();
	value_list = new Vector();

	edge_flag = false;
    }

    public AdjoiningPixel ( ImagePixy image, Point pt ) {
	point_list = new Vector();
	value_list = new Vector();

	edge_flag = false;

	// detected pixels will be cleared in original image
	construct_image = image;
	construct(new Point(pt.x, pt.y));
    }

    // recurrent process of constructor
    private void construct ( Point pt ) {
	if (0 <= pt.x  &&  pt.x < construct_image.getWidth()  &&
	    0 <= pt.y  &&  pt.y < construct_image.getHeight()  &&
	    construct_image.getValue(pt) > 0.0) {

	    point_list.addElement(pt);
	    value_list.addElement(new Double(construct_image.getValue(pt)));

	    // clear
	    construct_image.setValue(pt, 0.0);

	    if (pt.x == 0  ||  pt.x == construct_image.getWidth() - 1  ||  pt.y == 0  ||  pt.y == construct_image.getHeight() - 1)
		edge_flag = true;

	    construct(new Point(pt.x, pt.y - 1));
	    construct(new Point(pt.x - 1, pt.y));
	    construct(new Point(pt.x + 1, pt.y));
	    construct(new Point(pt.x, pt.y + 1));
	}
    }

    public int size ( ) {
	return point_list.size();
    }

    public boolean isEdge ( ) {
	return edge_flag;
    }

    public Point elementPointAt ( int i ) {
	return (Point)point_list.elementAt(i);
    }

    public double elementValueAt ( int i ) {
	Double f = (Double)value_list.elementAt(i);
	return f.doubleValue();
    }

    public void add ( Point pt, double value ) {
	point_list.addElement(pt);
	value_list.addElement(new Double(value));
    }

    public int findIndex ( Point pt ) {
	for (int i = 0 ; i < size() ; i++) {
	    if (elementPointAt(i).equals(pt))
		return i;
	}
	return -1;
    }

    public int findPeakIndex ( ) {
	int index = -1;
	double max = -1.0;
	for (int i = 0 ; i < size() ; i++) {
	    if (elementValueAt(i) > max) {
		index = i;
		max = elementValueAt(i);
	    }
	}
	return index;
    }

    // return index of (idx)th value
    // findSortedIndex(0) equals to findPeakIndex()
    public int findSortedIndex ( int idx ) {
	Array array = new Array(size());

	for (int i = 0 ; i < size() ; i++)
	    array.set(i, elementValueAt(i));

	ArrayIndex index = array.sortDescendant();

	if (idx < size())
	    return index.get(idx);

	return index.get(size() - 1);
    }

    // extract and remove pixels over threshold
    public AdjoiningPixel extract ( int peak_index, double threshold ) {
	AdjoiningPixel ap = new AdjoiningPixel();
	ap.edge_flag = edge_flag;

	extract_ap = ap;
	extract_index = peak_index;
	extract_threshold = threshold;
	extract_recurrent();

	return ap;
    }

    private void extract_recurrent ( ) {
	if (extract_index >= 0  &&  elementValueAt(extract_index) > extract_threshold) {
	    Point pt = elementPointAt(extract_index);

	    extract_ap.add(elementPointAt(extract_index), elementValueAt(extract_index));
	    point_list.removeElementAt(extract_index);
	    value_list.removeElementAt(extract_index);

	    extract_index = findIndex(new Point(pt.x, pt.y - 1));	extract_recurrent();
	    extract_index = findIndex(new Point(pt.x - 1, pt.y));	extract_recurrent();
	    extract_index = findIndex(new Point(pt.x + 1, pt.y));	extract_recurrent();
	    extract_index = findIndex(new Point(pt.x, pt.y + 1));	extract_recurrent();
	}
    }

    // check if two pixel groups are adjoining
    public boolean isAdjoining ( AdjoiningPixel ap ) {
	for (int i = 0 ; i < size() ; i++) {
	    Point pt1 = elementPointAt(i);
	    for (int j = 0 ; j < ap.size() ; j++) {
		Point pt2 = ap.elementPointAt(j);
		if (Math.abs(pt1.x - pt2.x) + Math.abs(pt1.y - pt2.y) <= 1)
		    return true;
	    }
	}
	return false;
    }

    public void append ( AdjoiningPixel ap ) {
	for (int i = 0 ; i < ap.size() ; i++) {
	    point_list.addElement(ap.elementPointAt(i));
	    value_list.addElement(new Double(ap.elementValueAt(i)));
	}
    }

    // deblend into stars
    // this will be destroyed
    public AdjoiningPixelSet deblend ( double noise ) {
	Vector ap_list = new Vector();

	while (size() > 0) {
	    // find peak position
	    int peak_index = findPeakIndex();
	    double peak_value = elementValueAt(peak_index);

	    // extract only pixels surrounding around the peak
	    double threshold = peak_value - noise * Math.sqrt(peak_value / noise + 1.0);
//	    double threshold = peak_value - noise * (Math.log(peak_value / noise + 1.0) + 1.0);
	    AdjoiningPixel adjoin_pixel = extract(peak_index, threshold);

	    // determine if a new peak or foot of already detected peak
	    int adjoin_count = 0;
	    int adjoin_index = -1;
	    for (int i = 0 ; i < ap_list.size() ; i++) {
		AdjoiningPixel ap = (AdjoiningPixel)ap_list.elementAt(i);
		if (adjoin_pixel.isAdjoining(ap)) {
		    adjoin_count++;
		    adjoin_index = i;
		}
	    }

	    if (adjoin_count == 0) {	// new peak
		ap_list.addElement(adjoin_pixel);
	    } else if (adjoin_count == 1) {	// foot of an already detected peak
		AdjoiningPixel ap = (AdjoiningPixel)ap_list.elementAt(adjoin_index);
		ap.append(adjoin_pixel);
	    } else {	// foot of already detected several peaks
		// ignore
	    }
	}

	AdjoiningPixelSet ap_set = new AdjoiningPixelSet();
	for (int i = 0 ; i < ap_list.size() ; i++)
	    ap_set.add((AdjoiningPixel)ap_list.elementAt(i));

	return ap_set;
    }
}
