/*
 * DetectList.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.*;
import java.util.Vector;
import misao.util.*;

public class DetectList extends StarList {
    public DetectList ( ) {
	super();
    }

    public DetectList ( StarList l ) {
	super(l);
    }

    // convert brightness (amount of pixel value) into magnitude according as catalog data.
    // assuming that detected list and catalog list are already sorted.
    // step: number of catalog data to calculate average for one detected star
    public int applyMagnitude ( StarList catalog_list, int step ) {
	return applyMagnitude(catalog_list, step, "");
    }
    public int applyMagnitude ( StarList catalog_list, int step, String mag_code ) {
	int applied_count = 0;

	for (int i = 0 ; i < size() ; i++) {
	    double mag = 0.0;
	    int count = 0;
	    for (int j = i * step ; j < (i + 1) * step  &&  j < catalog_list.size() ; j++) {
		mag += catalog_list.elementAt(j).getMag();
		count++;
	    }

	    if (count > 0) {
		if (mag_code.length() > 0)
		    ((DetectData)elementAt(i)).setMag(mag / (double)count, mag_code);
		else
		    elementAt(i).setMag(mag / (double)count);
		applied_count++;
	    } else {
		if (mag_code.length() > 0)
		    ((DetectData)elementAt(i)).setMag(99.9, mag_code);
		else
		    elementAt(i).setMag(99.9);
	    }
	}

	return applied_count;
    }

    // convert brightness to magnitude
    public void translateMagnitude ( MagnitudeTranslationFormula mag_formula ) {
	translateMagnitude(mag_formula, "");
    }
    public void translateMagnitude ( MagnitudeTranslationFormula mag_formula, String mag_code ) {
	for (int i = 0 ; i < size() ; i++) {
	    DetectData sd = (DetectData)elementAt(i);
	    double mag = mag_formula.calculate(sd.getValue());
	    if (mag_code.length() > 0)
		sd.setMag(mag, mag_code);
	    else
		sd.setMag(mag);
	}
    }

    // ascendant sort in order of value
    public void sortValueAscendant ( ) {
	Array value_array = new Array(size());

	for (int i = 0 ; i < size() ; i++)
	    value_array.set(i, ((DetectData)elementAt(i)).getValue());

	ArrayIndex index = value_array.sortAscendant();

	StarList newList = new StarList();
	for (int i = 0 ; i < size() ; i++)
	    newList.addElement(elementAt(index.get(i)));

	removeAllElements();
	add(newList);
    }

    // descendant sort in order of value
    public void sortValueDescendant ( ) {
	Array value_array = new Array(size());

	for (int i = 0 ; i < size() ; i++)
	    value_array.set(i, ((DetectData)elementAt(i)).getValue());

	ArrayIndex index = value_array.sortDescendant();

	StarList newList = new StarList();
	for (int i = 0 ; i < size() ; i++)
	    newList.addElement(elementAt(index.get(i)));

	removeAllElements();
	add(newList);
    }

    public void removeElementAtEdge ( ) {
	for (int i = 0 ; i < size() ; i++) {
	    if (((DetectData)elementAt(i)).isEdge()) {
		removeElementAt(i);
		i--;
	    }
	}
    }

    public void markCosmicRay ( double slope_threshold ) {
	for (int i = 0 ; i < size() ; i++) {
	    DetectData sd_detect = (DetectData)elementAt(i);
	    double slope = sd_detect.getPeak() / sd_detect.getValue();
	    if (slope >= slope_threshold) {
		sd_detect.markCosmicRay();
	    }
	}
    }

    public void rejectCosmicRay ( ) {
	for (int i = 0 ; i < size() ; i++) {
	    DetectData sd_detect = (DetectData)elementAt(i);
	    if (sd_detect.isCosmicRay()) {
		removeElementAt(i);
		i--;
	    }
	}
    }

    public void distribute ( ImagePixy image, ImagePixy sky_image, PsfDistribution psf_distribute ) {
	for (int i = 0 ; i < size() ; i++) {
	    DetectData sd = (DetectData)elementAt(i);
	    sd.setValue(0.0);
	}

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < image.getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < image.getWidth() ; pt.x++) {
		double value = image.getValue(pt) - sky_image.getValue(pt);
		double total_power = psf_distribute.getTotalPower(pt);
		if (total_power > 0.0) {
		    for (int i = 0 ; i < 3 ; i++) {
			DetectData sd = psf_distribute.getStarData(pt, i);
			if (sd != null)
			    sd.setValue(sd.getValue() + value * psf_distribute.getPower(pt, i) / total_power);
		    }
		}
	    }
	}
    }

    public void setPrintDetail ( ) {
	for (int i = 0 ; i < size() ; i++) {
	    DetectData sd = (DetectData)elementAt(i);
	    sd.setPrintDetail();
	}
    }

    public void setPrintSimple ( ) {
	for (int i = 0 ; i < size() ; i++) {
	    DetectData sd = (DetectData)elementAt(i);
	    sd.setPrintSimple();
	}
    }

    public void setPrintOnlyXYMag ( ) {
	for (int i = 0 ; i < size() ; i++) {
	    DetectData sd = (DetectData)elementAt(i);
	    sd.setPrintOnlyXYMag();
	}
    }
}
