/*
 * ImagePixy.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.*;
import java.io.*;
import java.util.Vector;
import misao.util.*;

public class ImagePixy extends ImageGray {
    public ImagePixy ( Size size ) throws Exception {
	super(size);
    }

    public ImagePixy ( ImageGray i ) {
	super(i);
    }

    public ImagePixy copy ( ) throws Exception {
	ImagePixy image = new ImagePixy(getSize());

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		image.setValue(pt, getValue(pt));
	    }
	}

	return image;
    }

    public int countBrighterThan ( double value ) {
	int count = 0;
	Point pt = new Point(0,0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		if (getValue(pt) > value)
		    count++;
	    }
	}
	return count;
    }

    // noise
    public double calculateNoise ( ImagePixy sky_image ) {
	int count = 0;
	double amount2 = 0;

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		if (getValue(pt) < sky_image.getValue(pt)) {
		    amount2 += (getValue(pt) - sky_image.getValue(pt)) * (getValue(pt) - sky_image.getValue(pt));
		    count++;
		}
	    }
	}
	amount2 /= (double)count;
	return Math.sqrt(amount2);	// average should be 0
    }

    // image of only stars
    public ImagePixy stellar ( ImagePixy sky_image, double noise ) throws Exception {
	ImagePixy image_stellar = new ImagePixy(getSize());

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		if (getValue(pt) < sky_image.getValue(pt) + noise) {
		    image_stellar.setValue(pt, 0.0);
		} else {
		    image_stellar.setValue(pt, getValue(pt) - sky_image.getValue(pt));
		}
	    }
	}

	return image_stellar;
    }

    // image of background (original - stellar image)
    public ImagePixy background ( ImagePixy sky_image, double noise ) throws Exception {
	ImagePixy image_stellar = new ImagePixy(getSize());

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		if (getValue(pt) < sky_image.getValue(pt) + noise) {
		    image_stellar.setValue(pt, getValue(pt));
		} else {
		    image_stellar.setValue(pt, sky_image.getValue(pt));
		}
	    }
	}

	return image_stellar;
    }

    // positive/nevative map for sky image
    public ImageGray binaryAtSky ( ImageGray sky_image ) throws Exception {
	ImageGray binary_image = new ImageGray(getSize());

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		if (getValue(pt) > sky_image.getValue(pt))
		    binary_image.setValue(pt, 1.0);
		else
		    binary_image.setValue(pt, 0.0);
	    }
	}

	return binary_image;
    }

    // equalize level of every row and column
    // roll vertical or horizontal white/dark line, especially being at edges
    public void equalizeRowColumn ( ) {
	// for each row
	Point pt = new Point(0, 0);
	Array array = new Array(getHeight());
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    Array array_row = new Array(getWidth());
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++)
		array_row.set(pt.x, getValue(pt));
	    array_row.sortAscendant();
	    array.set(pt.y, array_row.getValue(getWidth() / 2));
	}
	ArrayIndex index = array.sortAscendant();
	double mean = array.getValue(getHeight() / 2);
	double average = 0.0;
	double deviation = 0.0;
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    average += array.getValue(pt.y);
	    deviation += array.getValue(pt.y) * array.getValue(pt.y);
	}
	average /= (double)getHeight();
	deviation = Math.sqrt(deviation / (double)getHeight() - average * average);
	boolean illegal_flag = false;
	for (pt.y = getHeight() / 2 + 1 ; pt.y < getHeight() ; pt.y++) {
	    if (illegal_flag  ||  array.getValue(pt.y) - array.getValue(pt.y - 1) > deviation * 3.0) {
		illegal_flag = true;
		Point pt2 = new Point(0, index.get(pt.y));
		for (pt2.x = 0 ; pt2.x < getWidth() ; pt2.x++)
		    setValue(pt2, mean);
	    }
	}
	illegal_flag = false;
	for (pt.y = getHeight() / 2 - 1 ; pt.y >= 0 ; pt.y--) {
	    if (illegal_flag  ||  array.getValue(pt.y + 1) - array.getValue(pt.y) > deviation * 3.0) {
		illegal_flag = true;
		Point pt2 = new Point(0, index.get(pt.y));
		for (pt2.x = 0 ; pt2.x < getWidth() ; pt2.x++)
		    setValue(pt2, mean);
	    }
	}

	// for each column
	array = new Array(getWidth());
	for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
	    Array array_column = new Array(getHeight());
	    for (pt.y = 0 ; pt.y < getHeight() ; pt.y++)
		array_column.set(pt.y, getValue(pt));
	    array_column.sortAscendant();
	    array.set(pt.x, array_column.getValue(getHeight() / 2));
	}
	index = array.sortAscendant();
	mean = array.getValue(getWidth() / 2);
	average = 0.0;
	deviation = 0.0;
	for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
	    average += array.getValue(pt.x);
	    deviation += array.getValue(pt.x) * array.getValue(pt.x);
	}
	average /= (double)getWidth();
	deviation = Math.sqrt(deviation / (double)getWidth() - average * average);
	illegal_flag = false;
	for (pt.x = getWidth() / 2 + 1 ; pt.x < getWidth() ; pt.x++) {
	    if (illegal_flag  ||  array.getValue(pt.x) - array.getValue(pt.x - 1) > deviation * 3.0) {
		illegal_flag = true;
		Point pt2 = new Point(index.get(pt.x), 0);
		for (pt2.y = 0 ; pt2.y < getHeight() ; pt2.y++)
		    setValue(pt2, mean);
	    }
	}
	illegal_flag = false;
	for (pt.x = getWidth() / 2 - 1 ; pt.x >= 0 ; pt.x--) {
	    if (illegal_flag  ||  array.getValue(pt.x + 1) - array.getValue(pt.x) > deviation * 3.0) {
		illegal_flag = true;
		Point pt2 = new Point(index.get(pt.x), 0);
		for (pt2.y = 0 ; pt2.y < getHeight() ; pt2.y++)
		    setValue(pt2, mean);
	    }
	}
    }

    // detect stars
    // this should be stellar image (only stars)
    // this image will be cleared
    public AdjoiningPixelSet detect ( ) {
	AdjoiningPixelSet ap_set = new AdjoiningPixelSet();

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		if (getValue(pt) > 0.0) {
		    // detected pixels will be cleared
		    AdjoiningPixel adjoin_pixel = new AdjoiningPixel(this, pt);
		    ap_set.add(adjoin_pixel);
		}
	    }
	}

	return ap_set;
    }
}
