/*
 * MainFrame.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.*;
import misao.util.*;

public class MainFrame extends Frame implements Runnable {
    private Thread thread;

    protected EventManager event_manager;

    private PixyOutputData output_data;

    protected boolean view_coor_panel;
    private TextField textRA_h;
    private TextField textRA_m;
    private TextField textRA_s;
    private CheckboxGroup checkDeclSgn;
    private Checkbox checkDeclSgnP;
    private Checkbox checkDeclSgnM;
    private TextField textDecl_d;
    private TextField textDecl_m;
    private TextField textDecl_s;
    private Button btnMarkCoor;

    protected boolean view_XY_panel;
    private TextField textX;
    private TextField textY;
    private Button btnMarkXY;

    protected MainFrame ( ) {
	output_data = new PixyOutputData();

	event_manager = new EventManager();

	view_coor_panel = true;
	view_XY_panel = true;
    }

    public MainFrame ( PixyOutputData data ) throws Exception {
	this();

	output_data = data;

	PixyPairList pair_list = (PixyPairList)output_data.pair_list;

	pair_list.setStarColor();
//	pair_list.setStarPlotAsCircle();

	init();

	setTitle("PIXY system - Control");
	setSize(700, 80 + 25);
	setVisible(true);

	ImageFrame image_frame = new ImageFrame(output_data.image);
	image_frame.show();
	event_manager.add(image_frame);

	ChartFrame detect_frame = new ChartFrame("Detected Stars", new StarList(pair_list.createFirstList()), output_data.image.getSize(), new MapFunction(), new Position(0,0), output_data.image_info.faintest_mag);
	detect_frame.show();
	event_manager.add(detect_frame);

	ChartFrame catalog_frame = new ChartFrame("Catalog Data", new StarList(pair_list.createSecondList()), output_data.image.getSize(), output_data.map_function, new Position((double)output_data.image.getWidth() / 2.0, (double)output_data.image.getHeight() / 2.0), output_data.image_info.faintest_mag);
	catalog_frame.show();
	event_manager.add(catalog_frame);
    }

    public void init ( ) {
	textRA_h = new TextField(String.valueOf(output_data.image_info.center_coor.getRA_h()), 2);
	textRA_m = new TextField(String.valueOf(output_data.image_info.center_coor.getRA_m()), 2);
	textRA_s = new TextField(Astro.toString(output_data.image_info.center_coor.getRA_s(), 5, 2), 5);
	checkDeclSgn = new CheckboxGroup();
	checkDeclSgnP = new Checkbox("+", checkDeclSgn, output_data.image_info.center_coor.getDecl() >= 0.0);
	checkDeclSgnM = new Checkbox("-", checkDeclSgn, output_data.image_info.center_coor.getDecl() < 0.0);
	textDecl_d = new TextField(String.valueOf(output_data.image_info.center_coor.getAbsDecl_d()), 2);
	textDecl_m = new TextField(String.valueOf(output_data.image_info.center_coor.getAbsDecl_m()), 2);
	textDecl_s = new TextField(Astro.toString(output_data.image_info.center_coor.getAbsDecl_s(), 4, 2), 4);
	btnMarkCoor = new Button("Mark (R.A., Decl.)");

	textX = new TextField("0", 4);
	textY = new TextField("0", 4);
	btnMarkXY = new Button("Mark (x,y)");

	Panel panelCoor = new Panel(new FlowLayout());
	panelCoor.add(textRA_h);
	panelCoor.add(new Label("h"));
	panelCoor.add(textRA_m);
	panelCoor.add(new Label("m"));
	panelCoor.add(textRA_s);
	panelCoor.add(new Label("s"));
	panelCoor.add(checkDeclSgnP);
	panelCoor.add(checkDeclSgnM);
	panelCoor.add(textDecl_d);
	panelCoor.add(new Label("o"));
	panelCoor.add(textDecl_m);
	panelCoor.add(new Label("'"));
	panelCoor.add(textDecl_s);
	panelCoor.add(new Label("\""));
	panelCoor.add(btnMarkCoor);

	Panel panelXY = new Panel(new FlowLayout());
	panelXY.add(new Label("X:"));
	panelXY.add(textX);
	panelXY.add(new Label("Y:"));
	panelXY.add(textY);
	panelXY.add(btnMarkXY);

	setLayout(new BorderLayout());
	if (view_coor_panel)
	    add("North",  panelCoor);
	if (view_XY_panel)
	    add("Center", panelXY);
    }

    public void start ( ) {
	thread = new Thread(this);
	thread.start();
    }

    public void stop ( ) {
	if (thread != null) {
	    thread.stop();
	    thread = null;
	}
    }

    public boolean action (Event ev, Object obj) {
	if (ev.target instanceof Button) {
	    if (btnMarkCoor.getLabel().equals(obj)) {
		// point out the position of R.A. and Decl. user inputs
		try {
		    CoorMapping coor = new CoorMapping(Integer.parseInt(textRA_h.getText().trim()), Integer.parseInt(textRA_m.getText().trim()), (double)(Float.valueOf(textRA_s.getText().trim()).floatValue()), checkDeclSgnM.getState(), Integer.parseInt(textDecl_d.getText().trim()), Integer.parseInt(textDecl_m.getText().trim()), (double)(Float.valueOf(textDecl_s.getText().trim()).floatValue()));
		    Position pos = coor.mappingXY(output_data.image_info.center_coor);
		    pos.rescale((double)output_data.image.getWidth() / output_data.image_info.field_of_view);
		    pos = output_data.map_function.inverse().map(pos);
		    Position df = output_data.image_info.distortion_field.getValue(pos);
		    df.rescale(-1.0);
		    pos.add(df);
		    Point pt = new Point((int)(pos.getX() + (double)output_data.image.getWidth() / 2.0), (int)(pos.getY() + (double)output_data.image.getHeight() / 2.0));

		    event_manager.mark(pt);

		    return true;
		} catch ( NumberFormatException e ) {
		    System.err.println("Number format exception.");
		}
	    } else if (btnMarkXY.getLabel().equals(obj)) {
		// point out the position of R.A. and Decl. user inputs
		try {
		    Point pt = new Point(Integer.parseInt(textX.getText().trim()), Integer.parseInt(textY.getText().trim()));

		    event_manager.mark(pt);

		    return true;
		} catch ( NumberFormatException e ) {
		    System.err.println("Number format exception.");
		}
	    }
	}
	return false;
    }

    public void run ( ) {
	while (true) {
	    try {
		Thread.sleep(500);
	    } catch (InterruptedException e) {
		System.err.println("sleep() is interrupted. " + e.getMessage());
		break;
	    }
	}
    }
}
