/*
 * PSF.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import misao.util.*;

public class PSF {
    private double radius;

    private Position center_position;
    private double peak;
    private int count;

    private double r2logz;
    private double r4;

    // pixels required to calculate PSF by the least square method
    private static final int required_count = 5;

    public PSF ( PSF psf ) {
	radius = psf.radius;

	center_position = new Position(psf.center_position);
	peak = psf.peak;
	count = psf.count;
    }

    public PSF ( Position center, double p ) {
	center_position = center;
	peak = p;
	count = 0;

	r2logz = 0.0;
	r4 = 0.0;
    }

    public double getRadius ( ) {
	return radius;
    }

    public void add ( Position pos, double value ) {
	count++;

	double r2 = (pos.getX() - center_position.getX()) * (pos.getX() - center_position.getX()) + (pos.getY() - center_position.getY()) * (pos.getY() - center_position.getY());
	r2logz += r2 * Math.log(value / peak);
	r4 += r2 * r2;
    }

    public double solve ( ) {
	if (count >= required_count  &&  r2logz < 0.0  &&  r4 > 0.0) {
	    radius = Math.sqrt(- r4 / r2logz / 2.0);
	} else {
	    radius = 1.0;
	}

	return radius;
    }

    public double power ( Position pos ) {
	double r2 = (pos.getX() - center_position.getX()) * (pos.getX() - center_position.getX()) + (pos.getY() - center_position.getY()) * (pos.getY() - center_position.getY());
	return peak * Math.exp(- r2 / (2.0 * radius * radius));
    }
}
