/*
 * PairList.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.awt.Color;
import java.io.*;
import java.util.*;
import misao.util.*;
import misao.host.*;

public class PairList {
    // elements of the lists are references to those of the original lists
    protected Vector PS_list;
    protected Vector RM_list;
    protected Vector RP_list;
    protected Vector NP_list1;
    protected Vector NP_list2;
    protected Vector NR_list;

    private boolean output_statistics;
    private boolean output_all_data;

    private boolean output_PS_data;

    public PairList ( ) {
	PS_list = new Vector();
	RM_list = new Vector();
	RP_list = new Vector();
	NP_list1 = new Vector();
	NP_list2 = new Vector();
	NR_list = new Vector();

	output_statistics = true;
	output_all_data = true;

	output_PS_data = true;
    }

    public PairList ( PairList pl ) {
	this();

	for (int i = 0 ; i < pl.PS_list.size() ; i++)
	    PS_list.addElement(pl.PS_list.elementAt(i));
	for (int i = 0 ; i < pl.RM_list.size() ; i++)
	    RM_list.addElement(pl.RM_list.elementAt(i));
	for (int i = 0 ; i < pl.RP_list.size() ; i++)
	    RP_list.addElement(pl.RP_list.elementAt(i));
	for (int i = 0 ; i < pl.NP_list1.size() ; i++)
	    NP_list1.addElement(pl.NP_list1.elementAt(i));
	for (int i = 0 ; i < pl.NP_list2.size() ; i++)
	    NP_list2.addElement(pl.NP_list2.elementAt(i));
	for (int i = 0 ; i < pl.NR_list.size() ; i++)
	    NR_list.addElement(pl.NR_list.elementAt(i));
    }

    public PairList ( StarList list1, StarList list2, int pair_count ) {
	this();

	if (pair_count > list1.size())
	    pair_count = list1.size();
	if (pair_count > list2.size())
	    pair_count = list2.size();

	for (int i = 0 ; i < pair_count ; i++)
	    PS_list.addElement(new PairData(list1.elementAt(i), list2.elementAt(i), PairType.PS));

	for (int i = pair_count ; i < list1.size() ; i++)
	    NP_list1.addElement(new PairData(list1.elementAt(i), null));
	for (int i = pair_count ; i < list2.size() ; i++)
	    NP_list2.addElement(new PairData(null, list2.elementAt(i)));
    }

    public void outputPSData ( boolean f ) {
	output_PS_data = f;
    }

    public PairData elementPairAt ( int i ) {
	return (PairData)PS_list.elementAt(i);
    }

    public Vector getAllList ( ) {
	Vector l = new Vector();
	l.addElement(PS_list);
	l.addElement(RM_list);
	l.addElement(RP_list);
	l.addElement(NP_list1);
	l.addElement(NP_list2);
	l.addElement(NR_list);
	return l;
    }

    public int getPairCount ( ) {
	return PS_list.size();
    }

    public int getPairCountBrighterThan ( double mag ) {
	int count = 0;
	for (int i = 0 ; i < getPairCount() ; i++) {
	    PairData pair = elementPairAt(i);
	    double m = pair.getBrightestMag();
	    if (m != Astro.ILLEGAL_MAG  &&  m <= mag)
		count++;
	}
	return count;
    }

    public int getRemarkableCount ( ) {
	return getRemarkableMagCount() + getRemarkablePositionCount();
    }

    public int getRemarkableMagCount ( ) {
	return RM_list.size();
    }

    public int getRemarkablePositionCount ( ) {
	return RP_list.size();
    }

    public int getFirstListCount ( ) {
	return getPairCount() + getRemarkableCount() + NP_list1.size();
    }

    public int getSecondListCount ( ) {
	return getPairCount() + getRemarkableCount() + NP_list2.size();
    }

    public int getFirstNonpairListCount ( ) {
	return NP_list1.size();
    }

    public int getSecondNonpairListCount ( ) {
	return NP_list2.size();
    }

    public Vector createFirstList ( ) {
	return createFirstList(false);
    }
    public Vector createFirstList ( boolean nc_flag ) {
	Vector list = new Vector();
	for (int i = 0 ; i < PS_list.size() ; i++) {
	    PairData pair = (PairData)PS_list.elementAt(i);
	    list.addElement(pair.getFirstData());
	}
	for (int i = 0 ; i < RM_list.size() ; i++) {
	    PairData pair = (PairData)RM_list.elementAt(i);
	    list.addElement(pair.getFirstData());
	}
	for (int i = 0 ; i < RP_list.size() ; i++) {
	    PairData pair = (PairData)RP_list.elementAt(i);
	    list.addElement(pair.getFirstData());
	}
	for (int i = 0 ; i < NP_list1.size() ; i++) {
	    PairData pair = (PairData)NP_list1.elementAt(i);
	    list.addElement(pair.getFirstData());
	}
	if (nc_flag) {
	    for (int i = 0 ; i < NP_list2.size() ; i++) {
		PairData pair = (PairData)NP_list2.elementAt(i);
		list.addElement(pair.getFirstData());
	    }
	}
	return list;
    }
    public Vector createSecondList ( ) {
	return createSecondList(false);
    }
    public Vector createSecondList ( boolean nd_flag ) {
	Vector list = new Vector();
	for (int i = 0 ; i < PS_list.size() ; i++) {
	    PairData pair = (PairData)PS_list.elementAt(i);
	    list.addElement(pair.getSecondData());
	}
	for (int i = 0 ; i < RM_list.size() ; i++) {
	    PairData pair = (PairData)RM_list.elementAt(i);
	    list.addElement(pair.getSecondData());
	}
	for (int i = 0 ; i < RP_list.size() ; i++) {
	    PairData pair = (PairData)RP_list.elementAt(i);
	    list.addElement(pair.getSecondData());
	}
	if (nd_flag) {
	    for (int i = 0 ; i < NP_list1.size() ; i++) {
		PairData pair = (PairData)NP_list1.elementAt(i);
		list.addElement(pair.getSecondData());
	    }
	}
	for (int i = 0 ; i < NP_list2.size() ; i++) {
	    PairData pair = (PairData)NP_list2.elementAt(i);
	    list.addElement(pair.getSecondData());
	}
	return list;
    }

    public Vector createFirstPairList ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < PS_list.size() ; i++) {
	    PairData pair = (PairData)PS_list.elementAt(i);
	    list.addElement(pair.getFirstData());
	}
	return list;
    }
    public Vector createSecondPairList ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < PS_list.size() ; i++) {
	    PairData pair = (PairData)PS_list.elementAt(i);
	    list.addElement(pair.getSecondData());
	}
	return list;
    }

    public Vector createFirstNonpairList ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < NP_list1.size() ; i++) {
	    PairData pair = (PairData)NP_list1.elementAt(i);
	    list.addElement(pair.getFirstData());
	}
	return list;
    }
    public Vector createSecondNonpairList ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < NP_list2.size() ; i++) {
	    PairData pair = (PairData)NP_list2.elementAt(i);
	    list.addElement(pair.getSecondData());
	}
	return list;
    }

    // sort in order of magnitude
    public void sortPairListInFirstMag ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < PS_list.size() ; i++)
	    list.addElement(PS_list.elementAt(i));
	PS_list = new Vector();
	Array array = new Array(list.size());
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    array.set(i, pair.getFirstMag());
	}
	ArrayIndex index = array.sortAscendant();
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(index.get(i));
	    PS_list.addElement(pair);
	}
    }

    // sort in order of magnitude
    public void sortPairListInSecondMag ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < PS_list.size() ; i++)
	    list.addElement(PS_list.elementAt(i));
	PS_list = new Vector();
	Array array = new Array(list.size());
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    array.set(i, pair.getSecondMag());
	}
	ArrayIndex index = array.sortAscendant();
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(index.get(i));
	    PS_list.addElement(pair);
	}
    }

    // sort in order of magnitude
    public void sortRemarkableListInMag ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < RM_list.size() ; i++)
	    list.addElement(RM_list.elementAt(i));
	RM_list = new Vector();
	Array array = new Array(list.size());
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    array.set(i, pair.getBrightestMag());
	}
	ArrayIndex index = array.sortAscendant();
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(index.get(i));
	    RM_list.addElement(pair);
	}

	list = new Vector();
	for (int i = 0 ; i < RP_list.size() ; i++)
	    list.addElement(RP_list.elementAt(i));
	RP_list = new Vector();
	array = new Array(list.size());
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    array.set(i, pair.getBrightestMag());
	}
	index = array.sortAscendant();
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(index.get(i));
	    RP_list.addElement(pair);
	}
    }

    // sort in order of magnitude difference
    public void sortRemarkableListInMagnitudeDifference ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < RM_list.size() ; i++)
	    list.addElement(RM_list.elementAt(i));
	RM_list = new Vector();
	Array array = new Array(list.size());
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    array.set(i, Math.abs(pair.getFirstMag() - pair.getSecondMag()));
	}
	ArrayIndex index = array.sortDescendant();
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(index.get(i));
	    RM_list.addElement(pair);
	}

	list = new Vector();
	for (int i = 0 ; i < RP_list.size() ; i++)
	    list.addElement(RP_list.elementAt(i));
	RP_list = new Vector();
	array = new Array(list.size());
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    array.set(i, Math.abs(pair.getFirstMag() - pair.getSecondMag()));
	}
	index = array.sortDescendant();
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(index.get(i));
	    RP_list.addElement(pair);
	}
    }

    // sort in order of magnitude
    public void sortFirstNonpairListInMag ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < NP_list1.size() ; i++)
	    list.addElement(NP_list1.elementAt(i));
	NP_list1 = new Vector();
	Array array = new Array(list.size());
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    array.set(i, pair.getBrightestMag());
	}
	ArrayIndex index = array.sortAscendant();
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(index.get(i));
	    NP_list1.addElement(pair);
	}
    }
    public void sortSecondNonpairListInMag ( ) {
	Vector list = new Vector();
	for (int i = 0 ; i < NP_list2.size() ; i++)
	    list.addElement(NP_list2.elementAt(i));
	NP_list2 = new Vector();
	Array array = new Array(list.size());
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(i);
	    array.set(i, pair.getBrightestMag());
	}
	ArrayIndex index = array.sortAscendant();
	for (int i = 0 ; i < list.size() ; i++) {
	    PairData pair = (PairData)list.elementAt(index.get(i));
	    NP_list2.addElement(pair);
	}
    }

    protected PairType getFirstNonpairType ( ) {
	return PairType.NP1;
    }

    protected PairType getSecondNonpairType ( ) {
	return PairType.NP2;
    }

    public void addPairElement ( StarData data1, StarData data2 ) {
	PS_list.addElement(new PairData(data1, data2, PairType.PS));
    }

    public void addFirstNonpairElement ( StarData data ) {
	NP_list1.addElement(new PairData(data, null, PairType.NP1));
    }

    public void addSecondNonpairElement ( StarData data ) {
	NP_list2.addElement(new PairData(null, data, PairType.NP2));
    }

    public void appendFirstNonpairList ( StarList l ) {
	for (int i = 0 ; i < l.size() ; i++)
	    NP_list1.addElement(new PairData(l.elementAt(i), null, PairType.NP1));
    }

    public void appendSecondNonpairList ( StarList l ) {
	for (int i = 0 ; i < l.size() ; i++)
	    NP_list2.addElement(new PairData(null, l.elementAt(i), PairType.NP2));
    }

    public void addNonpairReference ( PairData pair ) {
	NR_list.addElement(pair);
    }

    // remove data fainter than the limiting magnitude
    public void removeDataFainterThan ( double limit ) {
	for (int i = 0 ; i < PS_list.size() ; i++) {
	    PairData pair = (PairData)PS_list.elementAt(i);
	    if (pair.getBrightestMag() >= limit)
		PS_list.removeElementAt(i--);
	}
	removeRemarkableDataFainterThan(limit);
	removeFirstNonpairDataFainterThan(limit);
	removeSecondNonpairDataFainterThan(limit);
    }
    // remove remarkable data fainter than the limiting magnitude
    public void removeRemarkableDataFainterThan ( double limit ) {
	for (int i = 0 ; i < RM_list.size() ; i++) {
	    PairData pair = (PairData)RM_list.elementAt(i);
	    if (pair.getBrightestMag() >= limit)
		RM_list.removeElementAt(i--);
	}
	for (int i = 0 ; i < RP_list.size() ; i++) {
	    PairData pair = (PairData)RP_list.elementAt(i);
	    if (pair.getBrightestMag() >= limit)
		RP_list.removeElementAt(i--);
	}
    }
    public void removeFirstNonpairDataFainterThan ( double limit ) {
	for (int i = 0 ; i < NP_list1.size() ; i++) {
	    PairData pair = (PairData)NP_list1.elementAt(i);
	    if (pair.getBrightestMag() >= limit)
		NP_list1.removeElementAt(i--);
	}
    }
    public void removeSecondNonpairDataFainterThan ( double limit ) {
	for (int i = 0 ; i < NP_list2.size() ; i++) {
	    PairData pair = (PairData)NP_list2.elementAt(i);
	    if (pair.getBrightestMag() >= limit)
		NP_list2.removeElementAt(i--);
	}
    }

    public void setAllNames ( ) {
	int PS_count = 1;
	int RM_count = 1;
	int RP_count = 1;
	int NP_count = 1;
	int NP1_count = 1;
	int NP2_count = 1;
	int NR_count = 1;

	Vector l = getAllList();
	for (int j = 0 ; j < l.size() ; j++) {
	    Vector list = (Vector)l.elementAt(j);
	    for (int i = 0 ; i < list.size() ; i++) {
		PairData pair = (PairData)list.elementAt(i);
		if (pair.type.isPairStar()) {
		    pair.setName("PS-" + Astro.toStringZeroPadding(PS_count++, 5));
		} else if (pair.type.isRemarkableMagnitude()) {
		    pair.setName("RM-" + Astro.toStringZeroPadding(RM_count++, 5));
		} else if (pair.type.isRemarkablePosition()) {
		    pair.setName("RP-" + Astro.toStringZeroPadding(RP_count++, 5));
		} else if (pair.type.isNonpairReference()) {
		    pair.setName("NR-" + Astro.toStringZeroPadding(NR_count++, 5));
		} else {
		    if (pair.type == PairType.NP1  ||  pair.type == PairType.ND)
			pair.setName("NP1-" + Astro.toStringZeroPadding(NP1_count++, 5));
		    else if (pair.type == PairType.NP2  ||  pair.type == PairType.NC)
			pair.setName("NP2-" + Astro.toStringZeroPadding(NP2_count++, 5));
		    else
			pair.setName("NP-" + Astro.toStringZeroPadding(NP_count++, 5));
		}
	    }
	}
    }

    public void setNRNames ( ) {
	int NR_count = 1;

	for (int i = 0 ; i < NR_list.size() ; i++) {
	    PairData pair = (PairData)NR_list.elementAt(i);
	    pair.setName("NR-" + Astro.toStringZeroPadding(NR_count++, 5));
	}
    }

    public void outputStatistics ( boolean f ) {
	output_statistics = f;
    }

    public void outputAllData ( boolean f ) {
	output_all_data = f;
    }

    public boolean isOutputStatistics ( ) {
	return output_statistics;
    }

    public boolean isOutputAllData ( ) {
	return output_all_data;
    }

    public void printStatistics ( PrintStream ps ) {
	new Parameter("PS", "" + PS_list.size()).print(ps);
	new Parameter("RM", "" + RM_list.size()).print(ps);
	new Parameter("RP", "" + RP_list.size()).print(ps);
	new Parameter("NP1", "" + NP_list1.size()).print(ps);
	new Parameter("NP2", "" + NP_list2.size()).print(ps);
	new Parameter("NR", "" + NR_list.size()).print(ps);
    }

    public void printData ( PrintStream ps ) {
	Vector l = getAllList();
	for (int j = 0 ; j < l.size() ; j++) {
	    Vector list = (Vector)l.elementAt(j);

	    if (! (output_PS_data == false  &&  list == PS_list)) {
		for (int i = 0 ; i < list.size() ; i++) {
		    PairData pair = (PairData)list.elementAt(i);
		    ps.println(pair.name);
		    Vector alias_list = pair.getAliasList();
		    for (int k = 0 ; k < alias_list.size() ; k++) {
			ReferenceName name = (ReferenceName)alias_list.elementAt(k);
			ps.println(name.getType().toString() + "=" + name.getName());
		    }
		    pair.print(ps);
		    ps.println("");
		}
	    }
	}
    }

    public void read ( DataInputStream dis ) throws Exception {
	String record = dis.readLine();
	while (record != null) {
	    if (record.length() > 0) {
		Vector list = new Vector();
		while (record != null  &&  record.length() > 0) {
		    list.addElement(record);
		    record = dis.readLine();
		}

		String[] string_list = new String[list.size()];
		for (int i = 0 ; i < list.size() ; i++)
		    string_list[i] = (String)list.elementAt(i);

		PairData pair = PairData.createPairData(string_list);
		if (pair == null)
		    pair = PixyPairData.createPairData(string_list);

		if (pair.type == PairType.PS) {
		    PS_list.addElement(pair);
		} else if (pair.type == PairType.RM) {
		    RM_list.addElement(pair);
		} else if (pair.type == PairType.RP) {
		    RP_list.addElement(pair);
		} else if (pair.type == PairType.NP1) {
		    NP_list1.addElement(pair);
		} else if (pair.type == PairType.NP2) {
		    NP_list2.addElement(pair);
		} else if (pair.type == PairType.NR) {
		    NR_list.addElement(pair);
		} else if (pair.type == getFirstNonpairType()) {
		    NP_list1.addElement(pair);
		} else if (pair.type == getSecondNonpairType()) {
		    NP_list2.addElement(pair);
		}
	    }

	    if (record != null)
		record = dis.readLine();
	}
    }
}
