/*
 * PixyPairData.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.pixy;

import java.io.*;
import java.util.*;
import misao.util.*;
import misao.host.*;

public class PixyPairData extends PairData {
    protected PixyPairData ( ) {
	super();
    }

    public PixyPairData ( StarData sd1, StarData sd2 ) {
	this(sd1, sd2, PairType.PS);
    }
    public PixyPairData ( StarData sd1, StarData sd2, PairType t ) {
	super(sd1, sd2, t);

//	if (sd1 instanceof DetectData) {
//	    if (((DetectData)sd1).isEdge())
//		addMemo("Detected star is at edge.");
//	}
    }

    public PixyPairData ( PairData pair1, PairData pair2 ) {
	super(pair1, pair2);
    }
    public PixyPairData ( PairData pair1, PairData pair2, PairType t ) {
	super(pair1, pair2, t);
    }

    protected String getFirstHeader ( ) {
	if (object2 instanceof DetectData)
	    return super.getFirstHeader();
	return "D|";
    }
    protected String getSecondHeader ( ) {
	if (object2 instanceof DetectData)
	    return super.getSecondHeader();
	return "C|";
    }

    // set (x,y) of a NC object
    public void setDetectPosition ( Position p ) {
	object1 = new DetectData(p);
    }

    // set (x,y) of a ND object on the other image
    public void setCatalogPosition ( Position p ) {
	object2 = new DetectData(p);
    }

    public static PairData createPairData ( String[] record ) {
	PixyPairData pair = new PixyPairData();

	pair.type = PairType.PS;

	Vector detect_list = new Vector();
	Vector catalog_list = new Vector();

	for (int i = 0 ; i < record.length ; i++) {
	    if (record[i].length() >= 2  &&  record[i].charAt(1) == '|') {
		switch (record[i].charAt(0)) {
		case 'D':
		    detect_list.addElement(record[i].substring(3));
		    break;
		case 'C':
		    catalog_list.addElement(record[i].substring(3));
		    break;
		case 'R':
		    pair.addReference(record[i].substring(3));
		    break;
		case 'M':
		    pair.addMemo(record[i].substring(3));
		    break;
		default:
		    return null;
		}
	    } else if (record[i].length() >= 2  &&  record[i].charAt(1) == '=') {
		pair.addAlias(new ReferenceName(record[i].substring(2), ReferenceType.getType(record[i].charAt(0))));
	    } else {
		pair.setName(record[i]);

		if (pair.name.substring(0,3).equals("PS-"))
		    pair.type = PairType.PS;
		else if (pair.name.substring(0,3).equals("RM-"))
		    pair.type = PairType.RM;
		else if (pair.name.substring(0,3).equals("RP-"))
		    pair.type = PairType.RP;
		else if (pair.name.substring(0,3).equals("ND-"))
		    pair.type = PairType.ND;
		else if (pair.name.substring(0,3).equals("NC-"))
		    pair.type = PairType.NC;
		else if (pair.name.substring(0,3).equals("NR-"))
		    pair.type = PairType.NR;
	    }
	}

	pair.object1 = null;
	pair.object2 = null;

	if (detect_list.size() > 0) {
	    DetectData sd_detect = null;
	    for (int i = 0 ; i < detect_list.size() ; i++) {
		String s = (String)detect_list.elementAt(i);
		if (i == 0)
		    sd_detect = DetectData.createDetectData(s);
		else
		    sd_detect.append(DetectData.createDetectData(s));
	    }
	    pair.object1 = sd_detect;
	}
	if (catalog_list.size() > 0) {
	    CatalogData sd_catalog = null;
	    for (int i = 0 ; i < catalog_list.size() ; i++) {
		String s = (String)catalog_list.elementAt(i);
		if (i == 0)
		    sd_catalog = CatalogData.createCatalogData(s);
		else
		    sd_catalog.append(CatalogData.createCatalogData(s));
	    }
	    pair.object2 = sd_catalog;
	}

	return pair;
    }
}
