/*
 * LD.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
cf. IBVS 3855, 4329, 4458, 4642, 4734
 *
 */

package misao.ref.catalog;

import java.util.*;
import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class LD extends ReferenceCatalog {
    private int IBVS_number;

    private int table_number;
    private int ld_number;
    private Vector string_list;

    public LD ( ) {
	IBVS_number = 0;
	table_number = 0;
	ld_number = 0;
	string_list = new Vector();
    }

    public ReferenceData createReferenceData ( String record ) {
	if (record.indexOf("Number 3855") > 0) {
	    IBVS_number = 3855;
	    return null;
	}
	if (record.indexOf("IBVShead{") > 0) {
	    int p = record.indexOf("IBVShead{");
	    IBVS_number = Integer.parseInt(record.substring(p+9, p+13));
	    return null;
	}

	String name = "";
	String dbvs_name = "";
	String RA = "";
	String Decl = "";
	String RA_Decl = "";
	String max = "";
	String min = "";
	String b_v = "";
	String type = "";
	String epoch = "";
	String period = "";

	if (IBVS_number == 3855) {
	    if (record.length() < 66  ||  ! record.substring(0,3).equals("LD ")  ||  record.charAt(6) != ' ')
		return null;

	    name = record.substring(0,6);
	    dbvs_name = "LD" + record.substring(3,6);
	    RA = record.substring(9,11) + " " + record.substring(11,13) + " " + record.substring(13,15);
	    Decl = record.substring(18,21) + " " + record.substring(21,25);
	    RA_Decl = "R.A. " + RA + " Decl. " + Decl + " (1950)";
	    max = record.substring(28,34).trim();
	    min = record.substring(34,41).trim();
	    b_v = record.substring(44,49).trim();
	    type = record.substring(51,57).trim();
	    epoch = record.substring(57,62).trim();
	    if (! epoch.equals("-"))
		epoch = "244" + epoch;
	    period = record.substring(62).trim();
	    if (record.length() >= 69)
		period = record.substring(62,69).trim();
	} else if (IBVS_number == 4329) {
	    if (record.length() < 3  ||  ! record.substring(0,3).equals("LD "))
		return null;

	    StringTokenizer st = new StringTokenizer(record, "&");

	    name = st.nextToken().trim();
	    dbvs_name = "LD" + name.substring(3);
	    RA = st.nextToken().trim();
	    Decl = st.nextToken().trim();
	    RA = Astro.replaceString(RA, "\\fh", "");
	    RA = Astro.replaceString(RA, "\\fmm", "");
	    RA = Astro.replaceString(RA, "\\fss", "");
	    Decl = Astro.replaceString(Decl, "\\degr", "");
	    Decl = Astro.replaceString(Decl, "\\farcm ", ".");
	    RA_Decl = "R.A. " + RA + " Decl. " + Decl + " (1950)";
	    max = st.nextToken().trim();
	    min = st.nextToken().trim();
	    b_v = st.nextToken().trim();
	    if (b_v.length() > 3  &&  b_v.charAt(0) == '$')
		b_v = b_v.substring(1,2) + b_v.substring(3);
	    type = st.nextToken().trim();
	    epoch = st.nextToken().trim();
	    if (epoch.charAt(0) != '-')
		epoch = "244" + epoch;
	     period = st.nextToken().trim();
	} else if (IBVS_number == 4458) {
	    if (record.length() >= 8  &&  record.substring(0,8).equals("{Table 1")) {
		table_number = 1;
		return null;
	    }
	    if (record.length() >= 8  &&  record.substring(0,8).equals("{Table 2")) {
		table_number = 2;
		return null;
	    }
	    if (record.length() >= 13  &&  record.substring(0,13).equals("\\end{tabular}")) {
		table_number = 0;
		return null;
	    }
	    if (record.length() >= 21  &&  record.substring(0,21).equals("\\centerline{Figure 1}")) {
		ld_number = 0;
		return null;
	    }

	    if (record.length() < 6  ||  ! record.substring(0,3).equals("LD "))
		return null;

	    int number = Integer.parseInt(record.substring(3,6));
	    if (number < ld_number + 1)
		return null;
	    ld_number = number;

	    if (table_number == 1) {
		string_list.addElement(record);
		return null;
	    } else if (table_number == 2) {
		String record2 = (String)string_list.elementAt(0);
		string_list.removeElementAt(0);

		StringTokenizer st = new StringTokenizer(record2, "&");

		name = st.nextToken().trim().substring(0,6);
		dbvs_name = "LD" + name.substring(3);
		RA = st.nextToken().trim();
		Decl = st.nextToken().trim();
		RA_Decl = "R.A. " + RA + " Decl. " + Decl + " (2000)";

		st = new StringTokenizer(record, "&");
		st.nextToken();
		String max_min = st.nextToken();
		max = max_min.substring(0, max_min.indexOf("~--~")).trim();
		min = max_min.substring(max_min.indexOf("~--~") + 4).trim();
		if (min.length() > 3  &&  min.charAt(0) == '$')
		    min = min.substring(1,2) + min.substring(3);
		b_v = st.nextToken().trim();
		if (b_v.length() > 3  &&  b_v.charAt(0) == '$')
		    b_v = b_v.substring(1,2) + b_v.substring(3);
		type = st.nextToken().trim();
		epoch = st.nextToken().trim();
		if (epoch.length() > 0)
		    epoch = "24" + epoch;
		period = st.nextToken().trim();
		period = period.substring(0, period.indexOf("\\\\")).trim();
	    } else {
		return null;
	    }
	} else if (IBVS_number == 4642) {
	    if (record.length() >= 38  &&  record.substring(0,38).equals("\\caption{Positions and identifications")) {
		table_number = 1;
		return null;
	    }
	    if (record.length() >= 30  &&  record.substring(0,30).equals("\\caption{Elements of variation")) {
		table_number = 2;
		return null;
	    }
	    if (record.length() >= 13  &&  record.substring(0,13).equals("\\end{tabular}")) {
		table_number = 0;
		return null;
	    }

	    if (record.length() < 6  ||  ! record.substring(0,3).equals("LD "))
		return null;

	    if (table_number == 1) {
		string_list.addElement(record);
		return null;
	    } else if (table_number == 2) {
		String record2 = (String)string_list.elementAt(0);
		string_list.removeElementAt(0);

		StringTokenizer st = new StringTokenizer(record2, "&");

		name = st.nextToken().trim().substring(0,6);
		dbvs_name = "LD" + name.substring(3);
		RA = st.nextToken().trim();
		RA = Astro.replaceString(RA, "\\phantom{.0}", ".0");
		Decl = st.nextToken().trim();
		RA_Decl = "R.A. " + RA + " Decl. " + Decl + " (2000)";

		st = new StringTokenizer(record, "&");
		st.nextToken();
		String max_min = st.nextToken();
		max = max_min.substring(0, max_min.indexOf("~--~")).trim();
		min = max_min.substring(max_min.indexOf("~--~") + 4).trim();
		if (min.length() > 3  &&  min.charAt(0) == '$') {
		    int p = min.substring(1).indexOf('$');
		    min = min.substring(1,2) + min.substring(p+2);
		}
		b_v = st.nextToken().trim();
		if (b_v.length() > 3  &&  b_v.charAt(0) == '$')
		    b_v = b_v.substring(1,2) + b_v.substring(3);
		type = st.nextToken().trim();
		epoch = st.nextToken().trim();
		if (epoch.length() > 0)
		    epoch = "24" + epoch;
		period = st.nextToken().trim();
		period = period.substring(0, period.indexOf("\\\\")).trim();
	    } else {
		return null;
	    }
	} else if (IBVS_number == 4734) {
	    if (record.length() >= 38  &&  record.substring(0,38).equals("\\caption{Positions and identifications")) {
		table_number = 1;
		return null;
	    }
	    if (record.length() >= 30  &&  record.substring(0,30).equals("\\caption{Elements of variation")) {
		table_number = 2;
		return null;
	    }
	    if (record.length() >= 13  &&  record.substring(0,13).equals("\\end{tabular}")) {
		table_number = 0;
		return null;
	    }

	    if (record.length() < 6  ||  ! record.substring(0,3).equals("LD "))
		return null;

	    if (table_number == 1) {
		string_list.addElement(record);
		return null;
	    } else if (table_number == 2) {
		String record2 = (String)string_list.elementAt(0);
		string_list.removeElementAt(0);

		StringTokenizer st = new StringTokenizer(record2, "&");

		name = st.nextToken().trim().substring(0,6);
		dbvs_name = "LD" + name.substring(3);
		RA = st.nextToken().trim();
		RA = Astro.replaceString(RA, "\\phantom{.0}", ".0");
		Decl = st.nextToken().trim();
		RA_Decl = "R.A. " + RA + " Decl. " + Decl + " (2000)";

		st = new StringTokenizer(record, "&");
		st.nextToken();
		max = st.nextToken().trim();
		min = st.nextToken().trim();
		if (min.length() > 3  &&  min.charAt(0) == '$')
		    min = min.substring(1,2) + min.substring(3);
		b_v = st.nextToken().trim();
		if (b_v.length() > 3  &&  b_v.charAt(0) == '$')
		    b_v = b_v.substring(1,2) + b_v.substring(3);
		type = st.nextToken().trim();
		epoch = st.nextToken().trim();
		if (epoch.length() > 0)
		    epoch = "24" + epoch;
		period = st.nextToken().trim();
		period = Astro.replaceString(period, "$\\le$", "<");
	    } else {
		return null;
	    }
	} else {
	    return null;
	}

	CoorMapping coor = new CoorMapping(Coor.createCoor(RA + " " + Decl));

	if (IBVS_number == 3855  ||  IBVS_number == 4329)
	    coor.precession(new JulianDay(Astro.BESSELL_1950), new JulianDay(Astro.JULIUS_2000));

	record = name + "  " + RA_Decl + "  m_v=" + max + "-" + min + "  b-v=" + b_v + "  type=" + type + "  epoch=" + epoch + "  period=" + period;

	ReferenceData data = new ReferenceData(name, dbvs_name, coor, record);

	return data;
    }
}
