/*
 * Mkn.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

/*
 * Data format sample
 * 
MKN    1   MCG+05-04-009 NGC  449  01 13.4 +32 50   27   16  15.50  SD12E
MKN    2   MCG+06-05-044           01 51.9 +36 40   25   20  14.00  S 12
 *
 */

package misao.ref.catalog;

import misao.pixy.*;
import misao.util.*;
import misao.host.*;
import misao.ref.*;

public class Mkn extends ReferenceCatalog {
    public Mkn ( ) {
    }

    public ReferenceData createReferenceData ( String record ) {
	String name = "MKN " + record.substring(3,9).trim();
	String dbvs_name = "MKN" + record.substring(3,8).trim();

	String alias1 = record.substring(11,24).trim();
	String alias2 = record.substring(25,34).trim();

	CoorMapping coor = new CoorMapping(Integer.parseInt(record.substring(35,37)), Integer.parseInt(record.substring(38,40)), ((double)Integer.parseInt(record.substring(41,42))) / 10.0 * 60.0, (record.charAt(43) == '-'), Integer.parseInt(record.substring(44,46)), Integer.parseInt(record.substring(47,49)), 0.0);
	coor.precession(new JulianDay(Astro.BESSELL_1950), new JulianDay(Astro.JULIUS_2000));

	String RA = Astro.toStringZeroPadding(coor.getRA_h(), 2) + " " + Astro.toStringZeroPadding((double)coor.getRA_m() + coor.getRA_s() / 60.0, 4, 2);
	String Decl = (coor.getDecl() < 0.0 ? '-' : '+') + Astro.toStringZeroPadding(coor.getAbsDecl_d(), 2) + " " + Astro.toStringZeroPadding((double)coor.getAbsDecl_m() + coor.getAbsDecl_s() / 60.0, 2, 2);

	String s = name + "  ";
	if (alias1.length() > 0)
	    s += alias1 + "  ";
	if (alias2.length() > 0)
	    s += alias2 + "  ";
	s += "R.A.=" + RA + "  Decl.=" + Decl + "  " + record.substring(61,66).trim() + " mag";

	ReferenceData data = new ReferenceData(name, dbvs_name, coor, s);

	return data;
    }
}
