/*
 * AreaManager.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

public class AreaManager {
    private Coor center_coor;
    private double DeclA, DeclB, DeclC, DeclN, DeclS, RAdiff;

    public AreaManager ( Coor coor, double fov ) {
	center_coor = coor;

	double decl = coor.getDecl();

	double sinb, cosb, sinC, cosC;

	if (decl >= 90.0)
	    decl = 89.99999;
	if (decl <= -90.0)
	    decl = -89.99999;

	if (decl >= 0.0) {
	    DeclA = decl + fov / 2.0;

	    cosb = Math.cos((90.0 - decl) * Astro.RAD) * Math.cos(fov * 0.7071067 * Astro.RAD) + Math.sin((90.0 - decl) * Astro.RAD) * Math.sin(fov * 0.7071067 * Astro.RAD) * 0.7071067;
	    sinb = Math.sqrt(1.0 - cosb * cosb);
	    sinC = Math.sin(fov * 0.7071067 * Astro.RAD) * 0.7071067 / sinb;
	    cosC = (Math.cos(fov * 0.7071067 * Astro.RAD) - Math.cos((90.0 - decl) * Astro.RAD) * cosb) / (Math.sin((90.0 - decl) * Astro.RAD) * sinb);
	    RAdiff = Math.atan2(sinC, cosC) / Astro.RAD;
	    DeclB = 90.0 - Math.atan2(sinb, cosb) / Astro.RAD;

	    cosb = Math.cos((90.0 - decl) * Astro.RAD) * Math.cos(fov * 0.7071067 * Astro.RAD) - Math.sin((90.0 - decl) * Astro.RAD) * Math.sin(fov * 0.7071067 * Astro.RAD) * 0.7071067;
	    sinb = Math.sqrt(1.0 - cosb * cosb);
	    DeclC = 90.0 - Math.atan2(sinb, cosb) / Astro.RAD;

	    DeclN = DeclA;
	    DeclS = DeclC;
	} else {
	    DeclA = decl - fov / 2.0;

	    cosb = Math.cos((90.0 - decl) * Astro.RAD) * Math.cos(fov * 0.7071067 * Astro.RAD) - Math.sin((90.0 - decl) * Astro.RAD) * Math.sin(fov * 0.7071067 * Astro.RAD) * 0.7071067;
	    sinb = Math.sqrt(1.0 - cosb * cosb);
	    sinC = Math.sin(fov * 0.7071067 * Astro.RAD) * 0.7071067 / sinb;
	    cosC = (Math.cos(fov * 0.7071067 * Astro.RAD) - Math.cos((90.0 - decl) * Astro.RAD) * cosb) / (Math.sin((90.0 - decl) * Astro.RAD) * sinb);
	    RAdiff = Math.atan2(sinC, cosC) / Astro.RAD;
	    DeclB = 90.0 - Math.atan2(sinb, cosb) / Astro.RAD;

	    cosb = Math.cos((90.0 - decl) * Astro.RAD) * Math.cos(fov * 0.7071067 * Astro.RAD) + Math.sin((90.0 - decl) * Astro.RAD) * Math.sin(fov * 0.7071067 * Astro.RAD) * 0.7071067;
	    sinb = Math.sqrt(1.0 - cosb * cosb);
	    DeclC = 90.0 - Math.atan2(sinb, cosb) / Astro.RAD;

	    DeclN = DeclC;
	    DeclS = DeclA;
	}
    }

    public boolean overlapArea ( Coor start_coor, Coor end_coor ) {
	double RAs = start_coor.getRA();
	double RAe = end_coor.getRA();
	double Decls = start_coor.getDecl();
	double Decle = end_coor.getDecl();

	if (RAs > RAe)
	    RAe += 360.0;
	if (Decle < Decls) {
	    double swap_tmp =Decls;
	    Decls = Decle;
	    Decle = swap_tmp;
	}

	if (DeclN >= Decls  &&  DeclS <= Decle) {
	    if ((DeclN >= 90  &&  DeclB <= Decle)  ||  (DeclS <= -90  &&  DeclB >= Decls)  ||
		(center_coor.getRA() - RAdiff <= RAe  &&  RAs <= center_coor.getRA() + RAdiff)  ||  (center_coor.getRA() - RAdiff <= RAe + 360.0  &&  RAs + 360 <= center_coor.getRA() + RAdiff)  ||  (center_coor.getRA() - RAdiff <= RAe - 360.0  &&  RAs - 360 <= center_coor.getRA() + RAdiff)) {
		return true;
	    }
	}

	return false;
    }

    public boolean inArea ( Coor coor ) {
	double diff_a = Math.abs(center_coor.getRA() - coor.getRA());
	if (diff_a > 180.0)
	    diff_a = 360.0 - diff_a;
	if (DeclN >= coor.getDecl()  &&  DeclS <= coor.getDecl()  &&
	    ((DeclN >= 90.0  &&  DeclB <= coor.getDecl())  ||  (DeclS <= -90.0  &&  DeclB >= coor.getDecl())  ||  diff_a < RAdiff)) {
	    return true;
	}

	return false;
    }
}
