/*
 * CatalogIndex.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

public class CatalogIndex {
    private static final Catalog catalog_list[] = {
	new Catalog("cdrom.gsc11", "[CD-ROM] Guide Star Catalog (ver. 1.1)", 17.0, 5.0, Catalog.CATALOG_CDROM, true, false),
	new Catalog("cdrom.usnoa10", "[CD-ROM] USNO-A1.0 (V=R+0.375(B-R))", 20.0, 5.0, Catalog.CATALOG_CDROM, true, false),
	new Catalog("cdrom.usnoa10r", "[CD-ROM] USNO-A1.0 (R)", 20.0, 5.0, Catalog.CATALOG_CDROM, false, false),
	new Catalog("cdrom.usnoa10b", "[CD-ROM] USNO-A1.0 (B)", 20.0, 5.0, Catalog.CATALOG_CDROM, false, false),
	new Catalog("cdrom.usnoa20", "[CD-ROM] USNO-A2.0 (V=R+0.375(B-R))", 20.0, 5.0, Catalog.CATALOG_CDROM, true, false),
	new Catalog("cdrom.usnoa20r", "[CD-ROM] USNO-A2.0 (R)", 20.0, 5.0, Catalog.CATALOG_CDROM, false, false),
	new Catalog("cdrom.usnoa20b", "[CD-ROM] USNO-A2.0 (B)", 20.0, 5.0, Catalog.CATALOG_CDROM, false, false),
	new Catalog("www.yale3", "[WWW] Yale Catalogue of Bright Stars (rev. 3)", 6.0, 60.0, Catalog.CATALOG_WWW, true, false),
	new Catalog("www.tycho", "[WWW] Hipparcos/Tycho Catalogue", 10.5, 60.0, Catalog.CATALOG_WWW, true, false),
	new Catalog("www.tycho_kyoto", "[WWW] Tycho Catalog (arranged version at Kyoto University)", 10.5, 60.0, Catalog.CATALOG_WWW, false, false),
	new Catalog("www.gsc12", "[WWW] Guide Star Catalog (ver. 1.2)", 17.0, 5.0, Catalog.CATALOG_WWW, true, false),
	new Catalog("www.usnoa20", "[WWW] USNO-A2.0 (V=R+0.375(B-R))", 20.0, 5.0, Catalog.CATALOG_WWW, true, false),
	new Catalog("www.usnoa20r", "[WWW] USNO-A2.0 (R)", 20.0, 5.0, Catalog.CATALOG_WWW, false, false),
	new Catalog("www.usnoa20b", "[WWW] USNO-A2.0 (R)", 20.0, 5.0, Catalog.CATALOG_WWW, false, false),
	new Catalog("www.usnoa10_vizier", "[WWW] USNO-A1.0 at VizieR (V=R+0.375(B-R))", 20.0, 5.0, Catalog.CATALOG_WWW, true, false),
	new Catalog("www.usnoa10r_vizier", "[WWW] USNO-A1.0 at VizieR (R)", 20.0, 5.0, Catalog.CATALOG_WWW, false, false),
	new Catalog("www.usnoa10b_vizier", "[WWW] USNO-A1.0 at VizieR (R)", 20.0, 5.0, Catalog.CATALOG_WWW, false, false),
	new Catalog("www.dss1", "[WWW] Digitized Sky Survey 1", 1.0, 99.9, Catalog.CATALOG_WWW, true, true)
    };

    private int index;

    public CatalogIndex ( ) {
	index = 0;
    }

    public static Catalog getCatalogOfCode ( String code ) {
	for (int i = 0 ; i < catalog_list.length ; i++) {
	    if (catalog_list[i].getCode().equals(code))
		return catalog_list[i];
	}
	return null;
    }

    public static Catalog getCatalogOfName ( String name ) {
	for (int i = 0 ; i < catalog_list.length ; i++) {
	    if (catalog_list[i].getName().equals(name))
		return catalog_list[i];
	}
	return null;
    }

    public Catalog getFirstStarCatalog ( ) {
	index = 0;
	return getNextStarCatalog();
    }

    public Catalog getNextStarCatalog ( ) {
	return getNextCatalog(-1, false, false);
    }

    public Catalog getFirstAllskyStarCatalog ( ) {
	index = 0;
	return getNextAllskyStarCatalog();
    }

    public Catalog getNextAllskyStarCatalog ( ) {
	return getNextCatalog(-1, true, false);
    }

    public Catalog getFirstAllskyCdromStarCatalog ( ) {
	index = 0;
	return getNextAllskyCdromStarCatalog();
    }

    public Catalog getNextAllskyCdromStarCatalog ( ) {
	return getNextCatalog(Catalog.CATALOG_CDROM, true, false);
    }

    public Catalog getFirstImageCatalog ( ) {
	index = 0;
	return getNextImageCatalog();
    }

    public Catalog getNextImageCatalog ( ) {
	return getNextCatalog(-1, false, true);
    }

    private Catalog getNextCatalog ( int type, boolean all_sky, boolean image ) {
	while (index < catalog_list.length) {
	    Catalog catalog = catalog_list[index];
	    if (type < 0  ||  catalog.getType() == type) {
		if (! all_sky  ||  catalog.isAllsky()) {
		    if ((image  &&  catalog.isImage())  ||  (! image  &&  ! catalog.isImage())) {
			index++;
			return catalog;
		    }
		}
	    }
	    index++;
	}
	return null;
    }
}
