/*
 * CdromListUSNO.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.io.*;
import java.util.Vector;

public abstract class CdromListUSNO extends CdromList {
    protected static final int USNO_V = 0;
    protected static final int USNO_R = 1;
    protected static final int USNO_B = 2;

    protected int mode;

    protected static final int default_buffer_size = 100000;

    protected abstract int getNumberOfFiles();

    protected abstract int getFileNumber ( int i );

    protected abstract int getDiskNumber ( int i );

    protected abstract String getVersion();

    public void setModeRmag ( ) {
	mode = USNO_R;
    }
    public void setModeBmag ( ) {
	mode = USNO_B;
    }

    public void load ( Coor center_coor, double fov, double mag_limit, String[] path_list ) throws Exception {
	load(center_coor, fov, mag_limit, path_list, mode);
    }

    public void loadRmag ( Coor center_coor, double fov, double mag_limit, String[] path_list ) throws Exception {
	load(center_coor, fov, mag_limit, path_list, USNO_R);
    }

    public void loadBmag ( Coor center_coor, double fov, double mag_limit, String[] path_list ) throws Exception {
	load(center_coor, fov, mag_limit, path_list, USNO_B);
    }

    protected void load ( Coor center_coor, double fov, double mag_limit, String[] path_list, int mag_type ) throws Exception {
	setUnit(fov);

	int i, j, k;
	String s;

	restrictArea(center_coor, fov);

	String filename;
	byte b_cat[] = new byte[12];
	byte b_acc[] = new byte[30];
	int ptr, count, seq;
	int ra, decl;
	int m, mr, mb;
	double mag_v, mag_r, mag_b, mag;

	for (i = 0 ; i < getNumberOfFiles() ; i++) {
	    Coor start_coor = new Coor(0.0, (double)getFileNumber(i) / 10.0 - 90.0);
	    Coor end_coor = new Coor(360.0, (double)(getFileNumber(i) + 75) / 10.0 - 90.0);

	    if (overlapArea(start_coor, end_coor)) {
		filename = Astro.toStringZeroPadding(getFileNumber(i), 4);

		File file_cat;
		file_cat = checkCdrom(path_list, "ZONE" + filename + ".CAT", "disc " + String.valueOf(getDiskNumber(i)));

		File file_acc;
		file_acc = checkCdrom(path_list, "ZONE" + filename + ".ACC", "disc " + String.valueOf(getDiskNumber(i)));

		byte[] b_data = new byte[default_buffer_size * 12];

		DataInputStream fin_acc = new DataInputStream(new BufferedInputStream(new FileInputStream(file_acc)));

		for (j = 0 ; j <= 2375 ; j += 25) {
		    fin_acc.readFully(b_acc, 0, 30);
		    s = new String(b_acc);

		    ptr = Integer.parseInt(s.substring(6,17).trim());
		    count = Integer.parseInt(s.substring(18,29).trim());

		    start_coor.setRA((double)(j*15) / 100.0);
		    end_coor.setRA((double)((j+25)*15) / 100.0);

		    if (overlapArea(start_coor, end_coor)) {
			FileInputStream fin_cat = new FileInputStream(file_cat);
			DataInputStream din_cat = new DataInputStream(new BufferedInputStream(fin_cat));
			for (k = 0 ; k < 12 ; k++)
			    din_cat.skipBytes(ptr-1);

			int loop_count = ((count - 1) / default_buffer_size) + 1;

			seq = ptr - 1;
			for (int loop = 0 ; loop < loop_count ; loop++) {
			    int read_size = default_buffer_size;
			    if (loop == loop_count - 1)
				read_size = count - default_buffer_size * (loop_count - 1);
			    din_cat.readFully(b_data, 0, read_size * 12);

			    for (k = 0 ; k < read_size ; k++) {
				seq++;

				m = Astro.intValueOf(b_data, k * 12 + 8, 4);
				if (m < 0)
				    m = - m;
				mr = m % 1000;
				mb = (int)(m / 1000);
				mb = mb % 1000;
				if (mr == 0)
				    mr = 999;
				if (mb == 0)
				    mb = 999;
				mag_r = (double)mr / 10.0;
				mag_b = (double)mb / 10.0;
				// preliminary V mag
				if (mr < 250) {
				    if (mb < 250)
					mag_v = Astro.getUSNOVmag(mag_r, mag_b);
				    else
					mag_v = mag_r;
				} else {
				    if (mb < 250)
					mag_v = mag_b;
				    else
					mag_v = 99.9;
				}

				if (mag_type == USNO_R) {
				    if (mr < 250)
					mag = mag_r;
				    else
					mag = mag_v;
				} else if (mag_type == USNO_B) {
				    if (mb < 250)
					mag = mag_b;
				    else
					mag = mag_v;
				} else {
				    mag = mag_v;
				}

				if (mag <= mag_limit) {
				    ra = Astro.intValueOf(b_data, k * 12, 4);
				    decl = Astro.intValueOf(b_data, k * 12 + 4, 4);

				    CoorMapping coor = new CoorMapping((double)ra / 100.0 / 3600.0, (double)decl / 100.0 / 3600.0 - 90.0);
				    if (inArea(coor)) {
					String id = Astro.toStringZeroPadding(seq, 8);
					id = "USNO-" + getVersion() + "_" + filename + "." + id;

					String RA = Astro.toStringZeroPadding(coor.getRA_h(), 2) + " " + Astro.toStringZeroPadding(coor.getRA_m(), 2) + " " + Astro.toStringZeroPadding(coor.getRA_s(), 6, 2);
					String Decl = (coor.getDecl() < 0.0 ? '-' : '+') + Astro.toStringZeroPadding(coor.getAbsDecl_d(), 2) + " " + Astro.toStringZeroPadding(coor.getAbsDecl_m(), 2) + " " + Astro.toStringZeroPadding(coor.getAbsDecl_s(), 5, 2);

					String record = null;
					if (isModeNull()) {
					    id = null;
					} else if (isModeSimple()) {
					    record = id + "   " + RA + "  " + Decl + "  ";
					    if (mr < 250)
						record += toStringMag(mr) + "R  ";
					    if (mb < 250)
						record += toStringMag(mb) + "B  ";
					    if (mr < 250  &&  mb < 250)
						record += "(" + Astro.toString(mag_v, 5, 2) + "V)";
					} else if (isModeDetail()) {
					    record = id + "  R.A.= " + RA + "  Decl.=" + Decl + "  ";
					    if (mr < 250)
						record += "Mag(R)=" + toStringMag(mr) + "  ";
					    if (mb < 250)
						record += "Mag(B)=" + toStringMag(mb) + "  ";
					    if (mr < 250  &&  mb < 250)
						record += "(preliminary V mag=" + Astro.toString(mag_v, 5, 2) + ")";
					}

					addStar(center_coor, coor, mag, id, record);
				    }
				}
			    }
			}

			din_cat.close();
		    }
		}

		fin_acc.close();
	    }
	}
    }

    protected static String toStringMag ( int m ) {
	if (m == 0) {
	    return " 0.0m";
	} else if (m == 999) {
	    return "99.9m";
	}

	return Astro.toString((double)m / 10.0, 4, 2) + " ";
    }

    public CatalogData createCatalogData ( String s ) {
	if (s.length() < 4  ||  ! s.substring(0,4).equals("USNO"))
	    return null;

	CatalogData data = new CatalogData();
	data.setRecord(s);

	int p = s.indexOf(' ');
	data.setID(s.substring(0,p));
	s = s.substring(p).trim();

	if (s.length() >= 5  &&  s.substring(0,5).equals("R.A.=")) {
	    p = s.indexOf("R.A.=");
	    s = s.substring(p+5);
	    String ra_decl = s;
	    p = s.indexOf("Decl.=");
	    s = s.substring(p+6);
	    ra_decl += " " + s;
	    data.setCoor(Coor.createCoor(s));

	    p = s.indexOf("preliminary V mag=");
	    if (p >= 0) {
		s = s.substring(p+18);
		data.setMag(Astro.doubleValueOf(s));
	    } else {
		p = s.indexOf("Mag(R)=");
		if (p >= 0) {
		    s = s.substring(p+7);
		    data.setMag(Astro.doubleValueOf(s));
		} else {
		    p = s.indexOf("Mag(B)=");
		    s = s.substring(p+7);
		    data.setMag(Astro.doubleValueOf(s));
		}
	    }
	} else {
	    data.setCoor(Coor.createCoor(s));
	    p = s.indexOf("  ");
	    s = s.substring(p).trim();
	    p = s.indexOf("  ");
	    s = s.substring(p).trim();
	    p = s.indexOf('(');
	    if (p >= 0) {
		int q = s.indexOf('V');
		data.setMag(Astro.doubleValueOf(s.substring(p+1,q)));
	    } else {
		data.setMag(Astro.doubleValueOf(s));
	    }
	}

	return data;
    }
}
