/*
 * CgiListHIPTYC.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

public class CgiListHIPTYC extends CgiList {
    protected static final String urlCGI = "http://astro.estec.esa.nl/hipparcos_scripts/HIPcatalogueSearch.pl";

    private static final int READ_HIP = 1;
    private static final int READ_TYC = 2;

    private StarList hip_list;

    public void load ( Coor center_coor, double fov, double mag ) throws Exception {
	setUnit(fov);

	String arg = "raDecim=" + center_coor.getRA() + "&decDecim=" + center_coor.getDecl() + "&box=" + fov + "&" + "raH=&raM=&raS=&decD=&decM=&raS=&boxD=&boxM=&boxS=&hipId=&hdId=&tyc1=&tyc2=&tyc3=";

	try {
	    openConnectionPost(urlCGI, arg);
	} catch ( Exception e ) {
	    throw e;
	}

	hip_list = new StarList();

	// Hipparcos
	read(READ_HIP, center_coor, fov, mag);

	// Tycho
	read(READ_TYC, center_coor, fov, mag);

	close();

	hip_list = null;
    }

    private void read ( int phase, Coor center_coor, double fov, double mag ) throws Exception {
	String record = null;
	double m;

	try {
	    record = readLine();
	} catch ( Exception e ) {
	    throw e;
	}
	while (record != null) {
	    if (record.length() >= 5  &&  record.substring(0,5).equals("<pre>"))
		break;

	    try {
		record = readLine();
	    } catch ( Exception e ) {
		throw e;
	    }
	}

	try {
	    record = readLine();
	} catch ( Exception e ) {
	    throw e;
	}
	while (record != null) {
	    if (record.length() < 300)
		break;

	    // when record data is not required, HIPPARCOS data will be ignored.
	    if (! (isModeNull()  &&  phase == READ_HIP)) {
		if (record.substring(1,76).equals("|<A href=\"http://astro.estec.esa.nl/hipparcos_scripts/HIPcatalogueSearch.pl")) {
		    int p = record.indexOf('>');
		    record = record.substring(p+1, record.length() - (p+1));

		    // R.A. and Decl.
		    CoorMapping coor = new CoorMapping(Integer.parseInt(record.substring(19,21)), Integer.parseInt(record.substring(22,24)), (double)(Float.valueOf(record.substring(25,30)).floatValue()), (record.charAt(31) == '-'), Integer.parseInt(record.substring(32,34)), Integer.parseInt(record.substring(35,37)), (double)(Float.valueOf(record.substring(38,42)).floatValue()));

		    m = (double)(Float.valueOf(record.substring(43,48).trim()).floatValue());

		    if (m <= mag) {
			String id = "";
			if (phase == READ_HIP) {
			    id = record.substring(6,12);
			    id = "HIP" + id.replace(' ', '0');
			} else {
			    id = record.substring(0,4) + "." + record.substring(6,10) + "." + record.substring(11,12);
			    id = "TYC" + id.replace(' ', '0');
			}

			record = id + "  R.A.=" + record.substring(19,21) + "h" + record.substring(22, 24) + "m" + record.substring(25,27) + "s" + record.substring(27,30) + "  Decl.=" + record.substring(31,34) + "o" + record.substring(35,37) + "'" + record.substring(38,40) + "\"" + record.substring(40,42) + "  V mag=" + record.substring(43,48);

			addStar(center_coor, coor, m, id, record);

			// unify HIP and TYC
			if (phase == READ_HIP) {
			    hip_list.addElement(elementAt(size() - 1));
			    removeElementAt(size() - 1);
			} else {
			    for (int i = 0 ; i < hip_list.size() ; i++) {
				CatalogData hip_data = (CatalogData)hip_list.elementAt(i);
				if (hip_data.getRecord().substring(11,47).equals(record.substring(16,52))) {
				    if (isModeNull())
					hip_data.setID(null);
				    elementAt(size() - 1).append(hip_data);
				    hip_list.removeElementAt(i);
				    break;
				}
			    }
			    if (isModeNull()) {
				CatalogData tyc_data = (CatalogData)elementAt(size() - 1);
				tyc_data.setID(null);
			    }
			}
		    }
		}
	    }

	    try {
		record = readLine();
	    } catch ( Exception e ) {
		throw e;
	    }
	}
    }
}
