/*
 * ImageFileFits.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.io.*;
import java.net.*;
import java.awt.Point;
import misao.contrib.imdisp.*;

public class ImageFileFits extends ImageGray {
    public ImageFileFits ( ImageGray i ) {
	super(i);
    }

    public ImageFileFits ( URL url ) throws Exception {
	URLConnection uc;
	try {
	    uc = url.openConnection();
	} catch ( IOException e ) {
	    System.err.println("Cannot connect to: " + url.toString());
	    throw e;
	}

	uc.connect();

	DataInputStream dis = null;
	try {
	    InputStream is = uc.getInputStream();
	    dis = new DataInputStream(new BufferedInputStream(is));
	} catch ( IOException e ) {
	    System.err.println("Cannot read from: " + url.toString());
	}

	FITSHeadEnv henv = new FITSHeadEnv(dis, false);

	int w = henv.width();
	int h = henv.height();

	Size size = new Size(w, h);

	try {
	    allocate(size);
	} catch ( Exception e ) {
	    throw e;
	}

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < size.getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < size.getWidth() ; pt.x++) {
		setValue(pt, henv.getNextPixel());
	    }
	}
	dis.close();
    }

/*
    public ImageFileFits ( URL url ) throws Exception {
	URLConnection uc;
	try {
	    uc = url.openConnection();
	} catch ( IOException e ) {
	    System.err.println("Cannot connect to: " + url.toString());
	    throw e;
	}

	uc.connect();

	DataInputStream dis = null;
	try {
	    InputStream is = uc.getInputStream();
	    dis = new DataInputStream(new BufferedInputStream(is));
	} catch ( IOException e ) {
	    System.err.println("Cannot read from: " + url.toString());
	}

	int bitpix = 8;
	double bzero = 0.0;
	double bscale = 1.0;
	int w = 0;
	int h = 0;

	for (int i = 0 ; i < 36 ; i++) {
	    StringBuffer sb = new StringBuffer();
	    int ch;
	    for (int j = 0 ; j < 80  &&  (ch = dis.read()) >= 0 ; j++) {
		sb.append((char)ch);
	    }
	    String s = sb.toString();

	    if (s.substring(0,6).equals("BITPIX")) {
		bitpix = Integer.parseInt(s.substring(10,30).trim());
	    } else if (s.substring(0,6).equals("NAXIS1")) {
		w = Integer.parseInt(s.substring(10,30).trim());
	    } else if (s.substring(0,6).equals("NAXIS2")) {
		h = Integer.parseInt(s.substring(10,30).trim());
	    } else if (s.substring(0,5).equals("BZERO")) {
		bzero = (double)(Float.valueOf(s.substring(10,30).trim()).floatValue());
	    } else if (s.substring(0,6).equals("BSCALE")) {
		bscale = (double)(Float.valueOf(s.substring(10,30).trim()).floatValue());
	    }
	}

	Size size = new Size(w, h);

	try {
	    allocate(size);
	} catch ( Exception e ) {
	    throw e;
	}

	// add-hock measure for MUTOH CCD controller for Windows
	// BZERO = -32767.0 -> +32768.0
	if (-32767.1 < bzero  &&  bzero < -32766.9)
	    bzero = 32768.0;

	int array_size = size.getWidth();
	if (bitpix == 16)
	    array_size *= 2;
	if (bitpix == 32)
	    array_size *= 4;
	byte b_array[] = new byte[array_size];

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < size.getHeight() ; pt.y++) {
	    dis.readFully(b_array, 0, array_size);

	    for (pt.x = 0 ; pt.x < size.getWidth() ; pt.x++) {
		if (bitpix == 8) {
		    int g = Astro.intValueOf(b_array, pt.x, 1);
		    setValue(pt, (double)g * bscale + bzero);
		} else if (bitpix == 16) {
		    int g = Astro.intValueOf(b_array, pt.x * 2, 2);
		    if ((g & (1 << 15)) != 0)
			g |= (65535 << 16);
		    setValue(pt, (double)g * bscale + bzero);
		    if ((double)g * bscale + bzero < 0.0)
			setValue(pt, (double)g * bscale + bzero + 65536.0);
		} else if (bitpix == 32) {
		    int g = Astro.intValueOf(b_array, pt.x * 4, 4);
		    setValue(pt, (double)g * bscale + bzero);
		}
	    }
	}
	dis.close();
    }
*/

    public void save ( File file ) throws Exception {
	try {
	    saveInteger16(file);
	} catch ( Exception e ) {
	    throw e;
	}
    }

    public void saveInteger16 ( File file ) throws Exception {
	PrintStream ps = null;
	try {
	    FileOutputStream fos = new FileOutputStream(file);
	    ps = new PrintStream(fos);
	} catch ( IOException e ) {
	    System.err.println("Cannot write to: " + file.toString());
	}

	ps.print("SIMPLE  =                    T                                                  ");
	ps.print("BITPIX  =                   16                                                  ");
	ps.print("NAXIS   =                    2                                                  ");
	ps.print("NAXIS1  = " + Astro.toString(getWidth(), 20) + "                                                  ");
	ps.print("NAXIS2  = " + Astro.toString(getHeight(), 20) + "                                                  ");
	ps.print("BZERO   =              32768.0                                                  ");
	ps.print("BSCALE  =                  1.0                                                  ");
	ps.print("END                                                                             ");
	for (int i = 0 ; i < 28 ; i++)
	    ps.print("                                                                                ");
	    
	MinMaxAverage mma = calculateMinMaxAverage();

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		int g = (int)((getValue(pt) - mma.getMin()) / (mma.getMax() - mma.getMin()) * 65536);
		if (g < 0)
		    g = 0;
		if (g > 65535)
		    g = 65535;
		g -= 32768;
		byte b = (byte)((g >> 8) & 255);
		ps.write(b);
		b = (byte)(g & 255);
		ps.write(b);
	    }
	}
	ps.close();
    }

    public MinMaxAverage getBestMinMax ( ) {
	MinMaxAverage mma = calculateMinMaxAverage();
	Histogram hist = new Histogram(10000);
	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		hist.addAt((int)((getValue(pt) - mma.getMin()) / (mma.getMax() - mma.getMin()) * 9999.9));
	    }
	}
	hist.accumulate();
	mma = new MinMaxAverage(hist.getAccumulation(getWidth() * getHeight() / 200) / 10000.0 * (mma.getMax() - mma.getMin()) + mma.getMin(), hist.getAccumulation(getWidth() * getHeight() * 199 / 200) / 10000.0 * (mma.getMax() - mma.getMin()) + mma.getMin(), 0.0, 1);
	return mma;
    }
}
