/*
 * ImageFilePgm.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.io.*;
import java.net.*;
import java.awt.Point;

public class ImageFilePgm extends ImageGray {
    public ImageFilePgm ( ImageGray i ) {
	super(i);
    }

    public ImageFilePgm ( URL url ) throws Exception {
	URLConnection uc;
	try {
	    uc = url.openConnection();
	} catch ( IOException e ) {
	    System.err.println("Cannot connect to: " + url.toString());
	    throw e;
	}

	uc.connect();

	DataInputStream dis = null;
	try {
	    InputStream is = uc.getInputStream();
	    dis = new DataInputStream(new BufferedInputStream(is));
	} catch ( IOException e ) {
	    System.err.println("Cannot read from: " + url.toString());
	}

	int w = 0;
	int h = 0;

	StringBuffer sb = new StringBuffer();
	int ch;
	int cr = 0;
	while ((ch = dis.read()) >= 0  &&  cr < 3) {
	    if (ch == '\n') {
		String s = sb.toString();
		if (s.charAt(0) != '#') {
		    cr++;
		    if (cr == 2) {
			int p = s.indexOf(' ');
			if (p < 0) {
			    w = Integer.parseInt(s);
			    h = w;
			} else {
			    w = Integer.parseInt(s.substring(0,p));
			    h = Integer.parseInt(s.substring(p+1));
			}
		    }
		}
		sb = new StringBuffer();
	    } else {
		sb.append((char)ch);
	    }
	}

	Size size = new Size(w, h);

	try {
	    allocate(size);
	} catch ( Exception e ) {
	    throw e;
	}

	byte b[] = new byte[size.getWidth()];

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < size.getHeight() ; pt.y++) {
	    try {
		dis.readFully(b, 0, size.getWidth());
	    } catch (EOFException e) {
	    }
	    for (pt.x = 0 ; pt.x < size.getWidth() ; pt.x++) {
		int p = b[pt.x];
		if (p < 0)
		    p += 256;
		setValue(pt, (double)p);
	    }
	}

	dis.close();
    }

    public void save ( File file ) throws Exception {
	PrintStream ps = null;
	try {
	    FileOutputStream fos = new FileOutputStream(file);
	    ps = new PrintStream(fos);
	} catch ( IOException e ) {
	    System.err.println("Cannot write to: " + file.toString());
	}

	ps.println("P5");
	ps.println("" + getWidth() + " " + getHeight());
	ps.println("255");

	MinMaxAverage mma = calculateMinMaxAverage();

	Point pt = new Point(0, 0);
	for (pt.y = 0 ; pt.y < getHeight() ; pt.y++) {
	    for (pt.x = 0 ; pt.x < getWidth() ; pt.x++) {
		ps.write(value256(getValue(pt), mma));
	    }
	}
	ps.close();
    }

    private byte value256 ( double val, MinMaxAverage mma ) {
	int g = (int)((val - mma.getMin()) / (mma.getMax() - mma.getMin()) * 256);
	if (g < 0)
	    g = 0;
	if (g > 255)
	    g = 255;
	byte b = (byte)(g & 255);
	return b;
    }
}
