/*
 * StarList.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.util.Vector;

public class StarList {
    private Vector list;

    private int sequential_counter;	// for use in getFirstElement(), getNextElement()

    public StarList ( ) {
	list = new Vector();
    }

    public StarList ( Vector l ) {
	this();

	for (int i = 0 ; i < l.size() ; i++) {
	    if (l.elementAt(i) instanceof StarData)
		list.addElement((StarData)l.elementAt(i));
	}
    }

    public StarList ( StarList sl ) {
	this();

	for (int i = 0 ; i < sl.size() ; i++) {
	    list.addElement(sl.elementAt(i));
	}
    }

    public StarList ( StarList sl, int n ) {
	this();

	if (n > sl.size())
	    n = sl.size();
	for (int i = 0 ; i < n ; i++) {
	    list.addElement(sl.elementAt(i));
	}
    }

    public StarList copy ( ) {
	StarList l = new StarList();
	for (int i = 0 ; i < size() ; i++) {
	    l.addElement(new StarData(elementAt(i)));
	}
	return l;
    }

    public void add ( StarList sl ) {
	for (int i = 0 ; i < sl.size() ; i++) {
	    list.addElement(sl.elementAt(i));
	}
    }

    public void addElement ( StarData data ) {
	list.addElement(data);
    }

    public int size ( ) {
	return list.size();
    }

    public StarData elementAt ( int i ) {
	return (StarData)list.elementAt(i);
    }

    public StarData elementLast ( ) {
	return (StarData)list.elementAt(list.size() - 1);
    }

    public void removeElementAt ( int i ) {
	list.removeElementAt(i);
    }

    public void removeLastElement ( ) {
	list.removeElementAt(list.size() - 1);
    }

    public void removeAllElements ( ) {
	list.removeAllElements();
    }

    public StarData getFirstElement ( ) {
	if (size() > 0) {
	    sequential_counter = 0;
	    return elementAt(0);
	}

	return null;
    }

    public StarData getNextElement ( ) {
	sequential_counter++;
	if (sequential_counter < size())
	    return elementAt(sequential_counter);

	return null;
    }

    public PositionList getPositionList ( ) {
	PositionList pl = new PositionList();
	for (int i = 0 ; i < size() ; i++) {
	    pl.addElement(elementAt(i).getPosition());
	}
	return pl;
    }

    // delete fainter objects than limit_mag
    public void limitMagAt ( double limit_mag ) {
	for (int i = 0 ; i < size() ; i++) {
	    if (elementAt(i).getMag() > limit_mag) {
		removeElementAt(i);
		i--;
	    }
	}
    }

    // add dx, dy to x, y
    public void shiftXY ( double dx, double dy ) {
	for (int i = 0 ; i < size() ; i++)
	    elementAt(i).shiftXY(new Position(dx, dy));
    }

    // mutiply x, y by w
    public void rescaleXY ( double w ) {
	for (int i = 0 ; i < size() ; i++)
	    elementAt(i).rescaleXY(w);
    }

    // ascendant sort in order of magnitude
    public void sortMagAscendant ( ) {
	Array mag_array = new Array(size());

	for (int i = 0 ; i < size() ; i++)
	    mag_array.set(i, elementAt(i).getMag());

	ArrayIndex index = mag_array.sortAscendant();

	Vector newList = new Vector();
	for (int i = 0 ; i < size() ; i++)
	    newList.addElement((StarData)list.elementAt(index.get(i)));
	list = newList;
    }

    // descendant sort in order of magnitude
    public void sortMagDescendant ( ) {
	Array mag_array = new Array(size());

	for (int i = 0 ; i < size() ; i++)
	    mag_array.set(i, elementAt(i).getMag());

	ArrayIndex index = mag_array.sortDescendant();

	Vector newList = new Vector();
	for (int i = 0 ; i < size() ; i++)
	    newList.addElement((StarData)list.elementAt(index.get(i)));
	list = newList;
    }
}
