/*
 * VariableStarName.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

import java.util.*;

public class VariableStarName {
    protected String dir;
    protected String file;

    protected VariableStarName ( ) {
    }

    public VariableStarName ( String name ) {
	String s = name.trim();
	int p = s.lastIndexOf(' ');
	if (p >= 0) {
	    String code = s.substring(p+1);
	    if (ConstellationTable.isConstellationCode(code)) {
		dir = code.toUpperCase();
		if (s.substring(0,p).equals("mu")  ||  s.substring(0,p).equals("nu"))
		    file = toFile(dir + "_" + s.substring(0,p));
		else
		    file = toFile(dir + s.substring(0,p));
		return;
	    }
	}

	if (s.length() >= 4  &&  s.substring(0,4).equalsIgnoreCase("IRAS")) {
	    dir = "iras";
	    file = toFile(s);

	    p = s.indexOf('+');
	    if (p < 0)
		p = s.indexOf('-');
	    dir += "/" + s.substring(p-5,p-3) + s.substring(p,p+2) + "0";

	    return;
	}

	if (s.length() >= 3  &&  s.substring(0,3).equalsIgnoreCase("NSV")) {
	    dir = "nsv";
	    file = toFile(s);
	    return;
	}

	if (s.length() >= 4  &&  s.substring(0,4).equalsIgnoreCase("MisV")) {
	    dir = "MisV";
	    file = toFile(s);
	    return;
	}

	if (s.length() >= 4  &&  s.substring(0,4).equalsIgnoreCase("MisE")) {
	    dir = "misao";
	    file = toFile(s);
	    return;
	}

	if (s.length() >= 4  &&  s.substring(0,4).equalsIgnoreCase("TmzV")) {
	    dir = "TmzV";
	    file = toFile(s);
	    return;
	}

	if (s.length() >= 4  &&  s.substring(0,4).equalsIgnoreCase("HadV")) {
	    dir = "HadV";
	    file = toFile(s);
	    return;
	}

	if (s.length() >= 4  &&  s.substring(0,4).equalsIgnoreCase("OGLE")) {
	    dir = "ogle";
	    file = toFile(s);
	    return;
	}

	if (s.length() >= 3  &&  s.substring(0,2).equalsIgnoreCase("HS")  &&
	    ('0' <= s.charAt(2)  &&  s.charAt(2) <= '9')) {
	    dir = "HS";
	    file = toFile(s);
	    return;
	}

	if (s.length() >= 5  &&  s.substring(0,4).equalsIgnoreCase("TASS")) {
	    dir = "TASS";
	    file = toFile(s.replace(' ','_'));
	    return;
	}

	if (s.length() >= 6  &&  s.substring(0,5).equalsIgnoreCase("ROTSE")) {
	    dir = "ROTSE";
	    file = toFile(s.replace(' ','_'));
	    return;
	}

	if (s.length() >= 3  &&  s.substring(0,3).equalsIgnoreCase("BeV")) {
	    dir = "BeV";
	    file = toFile(s);
	    return;
	}

	if (s.length() >= 5  &&  s.substring(0,5).equalsIgnoreCase("1RXSJ")) {
	    dir = "ROSAT";
	    file = toFile(s.replace(' ','_'));
	    return;
	}

	if (s.length() >= 6  &&  s.substring(0,2).equalsIgnoreCase("SN")) {
	    p = 2;
	    if (s.charAt(p) == ' ')
		p++;
	    if (('0' <= s.charAt(p)  &&  s.charAt(p) <= '9')  &&
		('0' <= s.charAt(p+1)  &&  s.charAt(p+1) <= '9')  &&
		('0' <= s.charAt(p+2)  &&  s.charAt(p+2) <= '9')  &&
		('0' <= s.charAt(p+3)  &&  s.charAt(p+3) <= '9')) {
		dir = "supernova";
		file = toFile(s);
		return;
	    }
	}

	if (s.length() >= 5  &&  s.substring(0,5).equals("Nova ")) {
	    StringTokenizer st = new StringTokenizer(s);
	    st.nextToken();
	    String cons = st.nextToken();
	    String s2 = cons.toUpperCase() + "nova" + st.nextToken();
	    if (st.hasMoreTokens()) {
		s2 += "-";
		while (st.hasMoreTokens()) {
		    String no = st.nextToken();
		    for (int i = 0 ; i < no.length() ; i++) {
			if ('0' <= no.charAt(i)  &&  no.charAt(i) <= '9')
			    s2 += no.charAt(i);
		    }
		}
	    }
	    file = toFile(s2);
	    if (ConstellationTable.isConstellationCode(cons))
		dir = cons.toUpperCase();
	    else
		dir = "others";
	    return;
	}

	dir = "others";
	file = toFile(s);
    }

    public String getDir ( ) {
	return dir;
    }

    public String getPath ( ) {
	return dir + "/" + file;
    }

    public static String toFile ( String s ) {
	String s2 = s;
	String s3 = "";
	for (int i = 0 ; i < s2.length() ; i++) {
	    if (s2.charAt(i) == ' '  ||  s2.charAt(i) == '\''  ||  s2.charAt(i) == '"'  ||  s2.charAt(i) == '('  ||  s2.charAt(i) == ')'  ||  s2.charAt(i) == '/'  ||  s2.charAt(i) == '\\'  ||  s2.charAt(i) == '$'  ||  s2.charAt(i) == '&'  ||  s2.charAt(i) == '`'  ||  s2.charAt(i) == '['  ||  s2.charAt(i) == ']'  ||  s2.charAt(i) == '?'  ||  s2.charAt(i) == '*'  ||  s2.charAt(i) == '<'  ||  s2.charAt(i) == '>'  ||  s2.charAt(i) == '|'  ||  s2.charAt(i) == '{'  ||  s2.charAt(i) == '}'  ||  s2.charAt(i) == ';'  ||  s2.charAt(i) == ':') {
	    } else {
		s3 += s2.charAt(i);
	    }
	}
	return s3;
    }
}
