/*
 * Xyz.java
 *
 * Copyright (C) 1998-2000 Seiichi Yoshida
 *
 */

package misao.util;

public class Xyz {
    private double x;
    private double y;
    private double z;

    public Xyz () {
	x = y = z = 0.0;
    }

    public Xyz ( double X, double Y, double Z ) {
	x = X;
	y = Y;
	z = Z;
    }

    public Xyz ( Position pos ) {
	x = pos.getX();
	y = pos.getY();
	z = 0.0;
    }

    public Xyz ( Position pos, double Z ) {
	x = pos.getX();
	y = pos.getY();
	z = Z;
    }

    public double getX ( ) {
	return x;
    }

    public double getY ( ) {
	return y;
    }

    public double getZ ( ) {
	return z;
    }

    public void set ( Xyz xyz ) {
	x = xyz.x;
	y = xyz.y;
	z = xyz.z;
    }

    public void add ( Xyz xyz ) {
	x += xyz.x;
	y += xyz.y;
	z += xyz.z;
    }

    public void rescale ( double r ) {
	x *= r;
	y *= r;
	z *= r;
    }

    // change precession
    public void precession (JulianDay jd_old, JulianDay jd_new) {
	double zeta,z,theta,sin_zt,cos_zt,sin_z,cos_z,sin_t,cos_t;
	double xx,xy,xz,yx,yy,yz,zx,zy,zz,x2,y2,z2;
	double ti,tf;

	ti = Astro.get_t2000(jd_old.getJD());
	tf = Astro.get_t2000(jd_new.getJD());
	if (jd_old.getJD() > jd_new.getJD()) {
	    double t_tmp = ti;
	    ti = tf;
	    tf = t_tmp;
	}
	tf -= ti;

	zeta = ((2306.2181+1.39656*ti-0.000139*ti*ti)*tf+ (0.30188-0.000344*ti)*tf*tf+0.017998*tf*tf*tf)*Astro.RAD/3600.0;
	z = zeta+((0.79280+0.000410*ti)*tf*tf+0.000205*tf*tf*tf)*Astro.RAD/3600.0;
	theta = ((2004.3109-0.85330*ti-0.000217*ti*ti)*tf- (0.42665+0.000217*ti)*tf*tf-0.041833*tf*tf*tf)*Astro.RAD/3600.0;
	if (jd_old.getJD() > jd_new.getJD()){
	    zeta = - zeta;
	    z = - z;
	    theta = - theta;
	}

	sin_zt = Math.sin(zeta);
	cos_zt = Math.cos(zeta);
	sin_z = Math.sin(z);
	cos_z = Math.cos(z);
	sin_t = Math.sin(theta);
	cos_t = Math.cos(theta);

	xx = cos_zt * cos_t * cos_z - sin_zt * sin_z;
	yx = - sin_zt * cos_t * cos_z - cos_zt * sin_z;
	zx = - sin_t * cos_z;
	xy = cos_zt * cos_t * sin_z + sin_zt * cos_z;
	yy = - sin_zt * cos_t * sin_z + cos_zt * cos_z;
	zy = - sin_t * sin_z;
	xz = cos_zt * sin_t;
	yz = - sin_zt * sin_t;
	zz = cos_t;

	x2 = xx * getX() + yx * getY() + zx * getZ();
	y2 = xy * getX() + yy * getY() + zy * getZ();
	z2 = xz * getX() + yz * getY() + zz * getZ();

	set(new Xyz(x2, y2, z2));
    }
}
